@@ -22,6 +22,94 @@ struct sax_parser_default_config
     static const uint8_t baseline_version = 10;
 };
 
+class sax_handler
+{
+public:
+    /**
+     * Called when a doctype declaration &lt;!DOCTYPE ... &gt; is encountered.
+     *
+     * @param param struct containing doctype declaration data.
+     */
+    void doctype(const orcus::sax::doctype_declaration& param)
+    {
+        (void)param;
+    }
+
+    /**
+     * Called when &lt;?... is encountered, where the '...' may be an
+     * arbitraray dentifier.  One common declaration is &lt;?xml which is
+     * typically given at the start of an XML stream.
+     *
+     * @param decl name of the identifier.
+     */
+    void start_declaration(const orcus::pstring& decl)
+    {
+        (void)decl;
+    }
+
+    /**
+     * Called when the closing tag (&gt;) of a &lt;?... ?&gt; is encountered.
+     *
+     * @param decl name of the identifier.
+     */
+    void end_declaration(const orcus::pstring& decl)
+    {
+        (void)decl;
+    }
+
+    /**
+     * Called at the start of each element.
+     *
+     * @param elem information of the element being parsed.
+     */
+    void start_element(const orcus::sax::parser_element& elem)
+    {
+        (void)elem;
+    }
+
+    /**
+     * Called at the end of each element.
+     *
+     * @param elem information of the element being parsed.
+     */
+    void end_element(const orcus::sax::parser_element& elem)
+    {
+        (void)elem;
+    }
+
+    /**
+     * Called when a segment of a text content is parsed.  Each text content
+     * is a direct child of an element, which may have multiple child contents
+     * when the element also has a child element that are direct sibling to
+     * the text contents or the text contents are splitted by a comment.
+     *
+     * @param val value of the text content.
+     * @param transient when true, the text content has been converted and is
+     *                  stored in a temporary buffer due to presence of one or
+     *                  more encoded characters, in which case <em>the passed
+     *                  text value needs to be either immediately converted to
+     *                  a non-text value or be interned within the scope of
+     *                  the callback</em>.
+     */
+    void characters(const orcus::pstring& val, bool transient)
+    {
+        (void)val; (void)transient;
+    }
+
+    /**
+     * Called upon parsing of an attribute of an element.  Note that <em>when
+     * the attribute's transient flag is set, the attribute value is stored in
+     * a temporary buffer due to presence of one or more encoded characters,
+     * and must be processed within the scope of the callback</em>.
+     *
+     * @param attr struct containing attribute information.
+     */
+    void attribute(const orcus::sax::parser_attribute& attr)
+    {
+        (void)attr;
+    }
+};
+
 /**
  * Template-based sax parser that doesn't use function pointer for
  * callbacks for better performance, especially on large XML streams.
@@ -34,6 +122,7 @@ public:
     typedef _Config config_type;
 
     sax_parser(const char* content, const size_t size, handler_type& handler);
+    sax_parser(const char* content, const size_t size, bool transient_stream, handler_type& handler);
     ~sax_parser();
 
     void parse();
@@ -63,7 +152,15 @@ private:
 template<typename _Handler, typename _Config>
 sax_parser<_Handler,_Config>::sax_parser(
     const char* content, const size_t size, handler_type& handler) :
-    sax::parser_base(content, size),
+    sax::parser_base(content, size, false),
+    m_handler(handler)
+{
+}
+
+template<typename _Handler, typename _Config>
+sax_parser<_Handler,_Config>::sax_parser(
+    const char* content, const size_t size, bool transient_stream, handler_type& handler) :
+    sax::parser_base(content, size, transient_stream),
     m_handler(handler)
 {
 }
@@ -79,7 +176,7 @@ void sax_parser<_Handler,_Config>::parse()
     m_nest_level = 0;
     mp_char = mp_begin;
     header();
-    blank();
+    skip_space_and_control();
     body();
 
     assert(m_buffer_pos == 0);
@@ -90,7 +187,7 @@ void sax_parser<_Handler,_Config>::header()
 {
     // we don't handle multi byte encodings so we can just skip bom entry if exists.
     skip_bom();
-    blank();
+    skip_space_and_control();
     if (!has_char() || cur_char() != '<')
         throw sax::malformed_xml_error("xml file must begin with '<'.", offset());
 
@@ -159,7 +256,7 @@ void sax_parser<_Handler,_Config>::element_open(std::ptrdiff_t begin_pos)
 
     while (true)
     {
-        blank();
+        skip_space_and_control();
         char c = cur_char();
         if (c == '/')
         {
@@ -255,7 +352,7 @@ void sax_parser<_Handler,_Config>::special_tag()
         {
             // check if this is a DOCTYPE.
             expects_next("OCTYPE", 6);
-            blank();
+            skip_space_and_control();
             if (has_char())
                 doctype();
         }
@@ -286,13 +383,13 @@ void sax_parser<_Handler,_Config>::declaration(const char* name_check)
     }
 
     m_handler.start_declaration(decl_name);
-    blank();
+    skip_space_and_control();
 
     // Parse the attributes.
     while (cur_char_checked() != '?')
     {
         attribute();
-        blank();
+        skip_space_and_control();
     }
     if (next_char_checked() != '>')
         throw sax::malformed_xml_error("declaration must end with '?>'.", offset());
@@ -332,7 +429,7 @@ void sax_parser<_Handler,_Config>::cdata()
         {
             // Found ']]>'.
             size_t cdata_len = i - 2;
-            m_handler.characters(pstring(p0, cdata_len), false);
+            m_handler.characters(pstring(p0, cdata_len), transient_stream());
             next();
             return;
         }
@@ -348,7 +445,7 @@ void sax_parser<_Handler,_Config>::doctype()
     // Parse the root element first.
     sax::doctype_declaration param;
     name(param.root_element);
-    blank();
+    skip_space_and_control();
 
     // Either PUBLIC or SYSTEM.
     size_t len = remains();
@@ -371,14 +468,14 @@ void sax_parser<_Handler,_Config>::doctype()
     }
 
     next_check();
-    blank();
+    skip_space_and_control();
     has_char_throw("DOCTYPE section too short.");
 
     // Parse FPI.
     value(param.fpi, false);
 
     has_char_throw("DOCTYPE section too short.");
-    blank();
+    skip_space_and_control();
     has_char_throw("DOCTYPE section too short.");
 
     if (cur_char() == '>')
@@ -396,7 +493,7 @@ void sax_parser<_Handler,_Config>::doctype()
     value(param.uri, false);
 
     has_char_throw("DOCTYPE section too short.");
-    blank();
+    skip_space_and_control();
     has_char_throw("DOCTYPE section too short.");
 
     if (cur_char() != '>')
@@ -426,7 +523,7 @@ void sax_parser<_Handler,_Config>::characters()
             buf.append(p0, mp_char-p0);
             characters_with_encoded_char(buf);
             if (buf.empty())
-                m_handler.characters(pstring(), false);
+                m_handler.characters(pstring(), transient_stream());
             else
                 m_handler.characters(pstring(buf.get(), buf.size()), true);
             return;
@@ -436,7 +533,7 @@ void sax_parser<_Handler,_Config>::characters()
     if (mp_char > p0)
     {
         pstring val(p0, mp_char-p0);
-        m_handler.characters(val, false);
+        m_handler.characters(val, transient_stream());
     }
 }
 
@@ -448,10 +545,11 @@ void sax_parser<_Handler,_Config>::attribute()
     attribute_name(attr.ns, attr.name);
 
 #if ORCUS_DEBUG_SAX_PARSER
-    std::ostringstream os;
-    os << "sax_parser::attribute: ns='" << attr.ns << "', name='" << attr.name << "'";
+    cout << "sax_parser::attribute: ns='" << attr.ns << "', name='" << attr.name << "'" << endl;
 #endif
 
+    skip_space_and_control();
+
     char c = cur_char();
     if (c != '=')
     {
@@ -460,15 +558,16 @@ void sax_parser<_Handler,_Config>::attribute()
         throw sax::malformed_xml_error(os.str(), offset());
     }
 
-    next_check();
+    next_check(); // skip the '='.
+    skip_space_and_control();
+
     attr.transient = value(attr.value, true);
     if (attr.transient)
         // Value is stored in a temporary buffer. Push a new buffer.
         inc_buffer_pos();
 
 #if ORCUS_DEBUG_SAX_PARSER
-    os << " value='" << attr.value << "'" << endl;
-    cout << os.str();
+    cout << "sax_parser::attribute: value='" << attr.value << "'" << endl;
 #endif
 
     m_handler.attribute(attr);
