@@ -14,6 +14,8 @@
 #include <mdds/global.hpp>
 #include <orcus/global.hpp>
 
+#include <sstream>
+
 using namespace std;
 
 namespace orcus {
@@ -148,6 +150,20 @@ double convert_centimeter(double value, length_unit_t unit_to)
     throw general_error("convert_centimeter: unsupported unit of measurement.");
 }
 
+double convert_millimeter(double value, length_unit_t unit_to)
+{
+    switch (unit_to)
+    {
+        case length_unit_t::twip:
+            // millimeters to twips : 25.4 mm = 1 inch = 1440 twips
+            return value / 25.4 * 1440.0;
+        default:
+            ;
+    }
+
+    throw general_error("convert_millimeter: unsupported unit of measurement.");
+}
+
 double convert_twip(double value, length_unit_t unit_to)
 {
     switch (unit_to)
@@ -182,6 +198,9 @@ double convert_xlsx_column_digit(double value, length_unit_t unit_to)
 
 double convert(double value, length_unit_t unit_from, length_unit_t unit_to)
 {
+    if (value == 0.0)
+        return value;
+
     switch (unit_from)
     {
         case length_unit_t::point:
@@ -190,6 +209,8 @@ double convert(double value, length_unit_t unit_from, length_unit_t unit_to)
             return convert_inch(value, unit_to);
         case length_unit_t::centimeter:
             return convert_centimeter(value, unit_to);
+        case length_unit_t::millimeter:
+            return convert_millimeter(value, unit_to);
         case length_unit_t::twip:
             return convert_twip(value, unit_to);
         case length_unit_t::xlsx_column_digit:
@@ -197,7 +218,12 @@ double convert(double value, length_unit_t unit_from, length_unit_t unit_to)
         default:
             ;
     }
-    throw general_error("convert: unsupported unit of measurement.");
+
+    std::ostringstream os;
+    os << "convert: unsupported unit of measurement (from "
+        << static_cast<int>(unit_from) << " to "
+        << static_cast<int>(unit_to) << ") (value=" << value << ")";
+    throw general_error(os.str());
 }
 
 }
