@@ -54,8 +54,8 @@ style_family_map::entry style_family_entries[] =
 
 odf_style_family to_style_family(const pstring& val)
 {
-    style_family_map style_family_map(style_family_entries, ORCUS_N_ELEMENTS(style_family_entries), style_family_unknown);
-    return style_family_map.find(val.get(), val.size());
+    static style_family_map map(style_family_entries, ORCUS_N_ELEMENTS(style_family_entries), style_family_unknown);
+    return map.find(val.get(), val.size());
 }
 
 class style_attr_parser : public std::unary_function<xml_token_attr_t, void>
@@ -111,26 +111,6 @@ public:
     const length_t& get_width() const { return m_width; }
 };
 
-class row_prop_attr_parser : public std::unary_function<xml_token_attr_t, void>
-{
-    length_t m_height;
-public:
-    void operator() (const xml_token_attr_t& attr)
-    {
-        if (attr.ns == NS_odf_style)
-        {
-            switch (attr.name)
-            {
-                case XML_row_height:
-                    m_height = to_length(attr.value);
-                break;
-            }
-        }
-    }
-
-    const length_t& get_height() const { return m_height; }
-};
-
 typedef mdds::sorted_string_map<spreadsheet::strikethrough_style_t> strikethrough_style_map;
 
 strikethrough_style_map::entry strikethrough_style_entries[] =
@@ -552,10 +532,9 @@ bool styles_context::can_handle_element(xmlns_id_t ns, xml_token_t /*name*/) con
 
 xml_context_base* styles_context::create_child_context(xmlns_id_t ns, xml_token_t /*name*/)
 {
-    if (ns == NS_odf_number )
+    if (ns == NS_odf_number)
     {
-        number_formatting_style* number_formatting = new number_formatting_style;
-        mp_child.reset(new number_formatting_context(get_session_context(), get_tokens(), m_styles, mp_styles, number_formatting));
+        mp_child.reset(new number_formatting_context(get_session_context(), get_tokens(), m_styles, mp_styles));
         mp_child->transfer_common(*this);
         return mp_child.get();
     }
@@ -567,7 +546,7 @@ void styles_context::end_child_context(xmlns_id_t /*ns*/, xml_token_t /*name*/,
 {
 }
 
-void styles_context::start_element(xmlns_id_t ns, xml_token_t name, const std::vector<xml_token_attr_t>& attrs)
+void styles_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_attrs_t& attrs)
 {
     xml_token_pair_t parent = push_stack(ns, name);
     if (ns == NS_odf_office)
@@ -612,12 +591,26 @@ void styles_context::start_element(xmlns_id_t ns, xml_token_t name, const std::v
             case XML_table_row_properties:
             {
                 xml_element_expected(parent, NS_odf_style, XML_style);
-                row_prop_attr_parser func;
-                func = std::for_each(attrs.begin(), attrs.end(), func);
+
+                std::for_each(attrs.begin(), attrs.end(),
+                    [&](const xml_token_attr_t& attr)
+                    {
+                        if (attr.ns == NS_odf_style)
+                        {
+                            switch (attr.name)
+                            {
+                                case XML_row_height:
+                                    m_current_style->row_data->height = to_length(attr.value);
+                                    m_current_style->row_data->height_set = true;
+                                    break;
+                            }
+                        }
+                    }
+                );
+
                 assert(m_current_style->family == style_family_table_row);
-                m_current_style->row_data->height = func.get_height();
+                break;
             }
-            break;
             case XML_table_properties:
                 xml_element_expected(parent, NS_odf_style, XML_style);
             break;
@@ -632,98 +625,8 @@ void styles_context::start_element(xmlns_id_t ns, xml_token_t name, const std::v
             }
             break;
             case XML_text_properties:
-            {
-                xml_element_expected(parent, NS_odf_style, XML_style);
-                if (mp_styles)
-                {
-                    text_prop_attr_parser func;
-                    func = std::for_each(attrs.begin(), attrs.end(), func);
-
-                    // Commit the font data.
-                    pstring font_name = func.get_font_name();
-                    if (!font_name.empty())
-                        mp_styles->set_font_name(font_name.get(), font_name.size());
-
-                    length_t font_size = func.get_font_size();
-                    if (font_size.unit == length_unit_t::point)
-                        mp_styles->set_font_size(font_size.value);
-
-                    if (func.is_bold())
-                        mp_styles->set_font_bold(true);
-
-                    if (func.is_italic())
-                        mp_styles->set_font_italic(true);
-
-                    if (func.has_color())
-                    {
-                        spreadsheet::color_elem_t red, green, blue;
-                        func.get_color(red, green, blue);
-                        mp_styles->set_font_color(0, red, green, blue);
-                    }
-
-                    if (func.has_underline())
-                    {
-                        if (func.underline_is_text_color() && func.has_color())
-                        {
-                            spreadsheet::color_elem_t red, green, blue;
-                            func.get_color(red, green, blue);
-                            mp_styles->set_font_underline_color(0, red, green, blue);
-                        }
-                        else
-                        {
-                            spreadsheet::color_elem_t red, green, blue;
-                            func.get_underline_color(red, green, blue);
-                            mp_styles->set_font_underline_color(0, red, green, blue);
-                        }
-                        spreadsheet::underline_width_t width = func.get_underline_width();
-                        mp_styles->set_font_underline_width(width);
-
-                        spreadsheet::underline_t style = func.get_underline_style();
-                        mp_styles->set_font_underline(style);
-
-                        spreadsheet::underline_type_t type = func.get_underline_type();
-                        mp_styles->set_font_underline_type(type);
-
-                        spreadsheet::underline_mode_t mode = func.get_underline_mode();
-                        mp_styles->set_font_underline_mode(mode);
-                    }
-                    if (func.has_strikethrough())
-                    {
-                        spreadsheet::strikethrough_style_t style = func.get_strikethrough_style();
-                        mp_styles->set_strikethrough_style(style);
-
-                        spreadsheet::strikethrough_width_t width = func.get_strikethrough_width();
-                        mp_styles->set_strikethrough_width(width);
-
-                        spreadsheet::strikethrough_type_t type = func.get_strikethrough_type();
-                        mp_styles->set_strikethrough_type(type);
-
-                        spreadsheet::strikethrough_text_t text = func.get_strikethrough_text();
-                        mp_styles->set_strikethrough_text(text);
-                    }
-
-                    size_t font_id = mp_styles->commit_font();
-
-                    switch (m_current_style->family)
-                    {
-                        case style_family_table_cell:
-                        {
-                            odf_style::cell* data = m_current_style->cell_data;
-                            data->font = font_id;
-                        }
-                        break;
-                        case style_family_text:
-                        {
-                            odf_style::text* data = m_current_style->text_data;
-                            data->font = font_id;
-                        }
-                        break;
-                        default:
-                            ;
-                    }
-                }
-            }
-            break;
+                start_text_properties(parent, attrs);
+                break;
             case XML_table_cell_properties:
             {
                 xml_element_expected(parent, NS_odf_style, XML_style);
@@ -829,10 +732,10 @@ bool styles_context::end_element(xmlns_id_t ns, xml_token_t name)
                     }
 
                     // ptr_map's first argument must be a non-const reference.
-                    pstring name = m_current_style->name;
+                    pstring style_name = m_current_style->name;
                     m_styles.insert(
                         odf_styles_map_type::value_type(
-                            name, std::move(m_current_style)));
+                            style_name, std::move(m_current_style)));
                     assert(!m_current_style);
                 }
             }
@@ -846,6 +749,109 @@ void styles_context::characters(const pstring& /*str*/, bool /*transient*/)
 {
 }
 
+void styles_context::start_text_properties(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
+{
+    static const xml_elem_stack_t expected = {
+        { NS_odf_style, XML_style },
+        { NS_odf_text, XML_list_level_style_number },
+        { NS_odf_text, XML_list_level_style_bullet },
+    };
+    xml_element_expected(parent, expected);
+
+    if (!mp_styles)
+        return;
+
+    if (parent != xml_token_pair_t(NS_odf_style, XML_style))
+        // TODO : handle this properly in the future.
+        return;
+
+    text_prop_attr_parser func;
+    func = std::for_each(attrs.begin(), attrs.end(), func);
+
+    // Commit the font data.
+    pstring font_name = func.get_font_name();
+    if (!font_name.empty())
+        mp_styles->set_font_name(font_name.get(), font_name.size());
+
+    length_t font_size = func.get_font_size();
+    if (font_size.unit == length_unit_t::point)
+        mp_styles->set_font_size(font_size.value);
+
+    if (func.is_bold())
+        mp_styles->set_font_bold(true);
+
+    if (func.is_italic())
+        mp_styles->set_font_italic(true);
+
+    if (func.has_color())
+    {
+        spreadsheet::color_elem_t red, green, blue;
+        func.get_color(red, green, blue);
+        mp_styles->set_font_color(0, red, green, blue);
+    }
+
+    if (func.has_underline())
+    {
+        if (func.underline_is_text_color() && func.has_color())
+        {
+            spreadsheet::color_elem_t red, green, blue;
+            func.get_color(red, green, blue);
+            mp_styles->set_font_underline_color(0, red, green, blue);
+        }
+        else
+        {
+            spreadsheet::color_elem_t red, green, blue;
+            func.get_underline_color(red, green, blue);
+            mp_styles->set_font_underline_color(0, red, green, blue);
+        }
+        spreadsheet::underline_width_t width = func.get_underline_width();
+        mp_styles->set_font_underline_width(width);
+
+        spreadsheet::underline_t style = func.get_underline_style();
+        mp_styles->set_font_underline(style);
+
+        spreadsheet::underline_type_t type = func.get_underline_type();
+        mp_styles->set_font_underline_type(type);
+
+        spreadsheet::underline_mode_t mode = func.get_underline_mode();
+        mp_styles->set_font_underline_mode(mode);
+    }
+    if (func.has_strikethrough())
+    {
+        spreadsheet::strikethrough_style_t style = func.get_strikethrough_style();
+        mp_styles->set_strikethrough_style(style);
+
+        spreadsheet::strikethrough_width_t width = func.get_strikethrough_width();
+        mp_styles->set_strikethrough_width(width);
+
+        spreadsheet::strikethrough_type_t type = func.get_strikethrough_type();
+        mp_styles->set_strikethrough_type(type);
+
+        spreadsheet::strikethrough_text_t text = func.get_strikethrough_text();
+        mp_styles->set_strikethrough_text(text);
+    }
+
+    size_t font_id = mp_styles->commit_font();
+
+    switch (m_current_style->family)
+    {
+        case style_family_table_cell:
+        {
+            odf_style::cell* data = m_current_style->cell_data;
+            data->font = font_id;
+        }
+        break;
+        case style_family_text:
+        {
+            odf_style::text* data = m_current_style->text_data;
+            data->font = font_id;
+        }
+        break;
+        default:
+            ;
+    }
+}
+
 void styles_context::commit_default_styles()
 {
     if (!mp_styles)
