@@ -23,6 +23,7 @@
 #include <mdds/sorted_string_map.hpp>
 
 using namespace std;
+namespace ss = orcus::spreadsheet;
 
 namespace orcus {
 
@@ -87,51 +88,7 @@ private:
     pstring m_name;
 };
 
-class column_attr_parser : public unary_function<xml_token_attr_t, void>
-{
-    pstring m_style_name;
-public:
-    void operator() (const xml_token_attr_t& attr)
-    {
-        if (attr.ns == NS_odf_table)
-        {
-            if (attr.name == XML_style_name)
-                m_style_name = attr.value;
-        }
-    }
-
-    const pstring& get_style_name() const { return m_style_name; }
-};
-
-class row_attr_parser : public unary_function<xml_token_attr_t, void>
-{
-    long m_number_rows_repeated;
-    pstring m_style_name;
-
-public:
-    row_attr_parser() : m_number_rows_repeated(1) {}
-
-    void operator() (const xml_token_attr_t& attr)
-    {
-        if (attr.ns == NS_odf_table)
-        {
-            switch (attr.name)
-            {
-                case XML_number_rows_repeated:
-                    m_number_rows_repeated = to_long(attr.value);
-                break;
-                case XML_style_name:
-                    m_style_name = attr.value;
-                break;
-            }
-        }
-    }
-
-    long get_number_rows_repeated() const { return m_number_rows_repeated; }
-    const pstring& get_style_name() const { return m_style_name; }
-};
-
-class cell_attr_parser : public unary_function<xml_token_attr_t, void>
+class cell_attr_parser
 {
 public:
     cell_attr_parser(session_context& cxt, ods_content_xml_context::cell_attr& attr) :
@@ -215,14 +172,18 @@ private:
         switch (attr.name)
         {
             case XML_style_name:
+            {
                 m_attr.style_name = attr.value;
-            break;
+                if (attr.transient)
+                    m_attr.style_name = m_cxt.m_string_pool.intern(m_attr.style_name).first;
+                break;
+            }
             case XML_number_columns_repeated:
                 m_attr.number_columns_repeated = to_long(attr.value);
-            break;
+                break;
             case XML_formula:
                 process_formula(attr.value, attr.transient);
-            break;
+                break;
             default:
                 ;
         }
@@ -259,10 +220,52 @@ private:
     ods_content_xml_context::cell_attr& m_attr;
 };
 
+void pick_up_named_range_or_expression(
+    session_context& cxt, const xml_attrs_t& attrs, xmlns_id_t exp_attr_ns, xml_token_t exp_attr_name,
+    ods_session_data::named_exp_type name_type, ss::sheet_t scope)
+{
+    pstring name;
+    pstring expression;
+    pstring base;
+
+    for (const xml_token_attr_t& attr : attrs)
+    {
+        if (attr.ns == exp_attr_ns && attr.name == exp_attr_name)
+        {
+            expression = cxt.intern(attr);
+            continue;
+        }
+
+        switch (attr.name)
+        {
+            case XML_name:
+                name = cxt.intern(attr);
+                break;
+            case XML_base_cell_address:
+                base = cxt.intern(attr);
+                break;
+        }
+    }
+
+    ods_session_data& ods_data = static_cast<ods_session_data&>(*cxt.mp_data);
+
+    if (!name.empty() && !expression.empty() && !base.empty())
+        ods_data.m_named_exps.emplace_back(name, expression, base, name_type, scope);
+}
+
 }
 
 // ============================================================================
 
+ods_content_xml_context::sheet_data::sheet_data() :
+    sheet(nullptr), index(-1) {}
+
+void ods_content_xml_context::sheet_data::reset()
+{
+    sheet = nullptr;
+    index = -1;
+}
+
 ods_content_xml_context::row_attr::row_attr() :
     number_rows_repeated(1)
 {
@@ -285,7 +288,8 @@ ods_content_xml_context::ods_content_xml_context(session_context& session_cxt, c
     m_para_index(0),
     m_has_content(false),
     m_styles(),
-    m_child_para(session_cxt, tokens, factory->get_shared_strings(), m_styles)
+    m_child_para(session_cxt, tokens, factory->get_shared_strings(), m_styles),
+    m_child_dde_links(session_cxt, tokens)
 {
     spreadsheet::iface::import_global_settings* gs = mp_factory->get_global_settings();
     if (gs)
@@ -307,6 +311,9 @@ bool ods_content_xml_context::can_handle_element(xmlns_id_t ns, xml_token_t name
     if (ns == NS_odf_office && name == XML_automatic_styles)
         return false;
 
+    if (ns == NS_odf_table && name == XML_dde_links)
+        return false;
+
     return true;
 }
 
@@ -326,6 +333,13 @@ xml_context_base* ods_content_xml_context::create_child_context(xmlns_id_t ns, x
         return mp_child.get();
     }
 
+    if (ns == NS_odf_table && name == XML_dde_links)
+    {
+        m_child_dde_links.reset();
+        m_child_dde_links.transfer_common(*this);
+        return &m_child_dde_links;
+    }
+
     return nullptr;
 }
 
@@ -405,9 +419,9 @@ void ods_content_xml_context::start_element(xmlns_id_t ns, xml_token_t name, con
         switch (name)
         {
             case XML_body:
-            break;
+                break;
             case XML_spreadsheet:
-            break;
+                break;
             default:
                 warn_unhandled();
         }
@@ -417,27 +431,62 @@ void ods_content_xml_context::start_element(xmlns_id_t ns, xml_token_t name, con
         switch (name)
         {
             case XML_calculation_settings:
-            break;
+                break;
             case XML_null_date:
                 xml_element_expected(parent, NS_odf_table, XML_calculation_settings);
                 start_null_date(attrs);
-            break;
+                break;
             case XML_table:
-                xml_element_expected(parent, NS_odf_office, XML_spreadsheet);
-                start_table(attrs);
-            break;
+                start_table(parent, attrs);
+                break;
             case XML_table_column:
-                xml_element_expected(parent, NS_odf_table, XML_table);
+            {
+                static const xml_elem_set_t expected = {
+                    { NS_odf_table, XML_table },
+                    { NS_odf_table, XML_table_column_group },
+                    { NS_odf_table, XML_table_columns },
+                    { NS_odf_table, XML_table_header_columns }
+                };
+                xml_element_expected(parent, expected);
                 start_column(attrs);
-            break;
+                break;
+            }
             case XML_table_row:
-                xml_element_expected(parent, NS_odf_table, XML_table);
+            {
+                static const xml_elem_set_t expected = {
+                    { NS_odf_table, XML_table },
+                    { NS_odf_table, XML_table_header_rows },
+                    { NS_odf_table, XML_table_row_group },
+                };
+                xml_element_expected(parent, expected);
                 start_row(attrs);
-            break;
+                break;
+            }
             case XML_table_cell:
                 xml_element_expected(parent, NS_odf_table, XML_table_row);
                 start_cell(attrs);
-            break;
+                break;
+            case XML_dde_links:
+                xml_element_expected(parent, NS_odf_office, XML_spreadsheet);
+                break;
+            case XML_dde_link:
+                xml_element_expected(parent, NS_odf_table, XML_dde_links);
+                break;
+            case XML_named_expressions:
+            {
+                static const xml_elem_set_t expected = {
+                    { NS_odf_office, XML_spreadsheet },
+                    { NS_odf_table, XML_table },
+                };
+                xml_element_expected(parent, expected);
+                break;
+            }
+            case XML_named_range:
+                start_named_range(parent, attrs);
+                break;
+            case XML_named_expression:
+                start_named_expression(parent, attrs);
+                break;
             default:
                 warn_unhandled();
         }
@@ -453,10 +502,10 @@ bool ods_content_xml_context::end_element(xmlns_id_t ns, xml_token_t name)
         switch (name)
         {
             case XML_body:
-            break;
+                break;
             case XML_spreadsheet:
                 end_spreadsheet();
-            break;
+                break;
             default:
                 ;
         }
@@ -466,21 +515,27 @@ bool ods_content_xml_context::end_element(xmlns_id_t ns, xml_token_t name)
         switch (name)
         {
             case XML_calculation_settings:
-            break;
+                break;
             case XML_null_date:
-            break;
+                break;
             case XML_table:
                 end_table();
-            break;
+                break;
             case XML_table_column:
                 end_column();
-            break;
+                break;
             case XML_table_row:
                 end_row();
-            break;
+                break;
             case XML_table_cell:
                 end_cell();
-            break;
+                break;
+            case XML_named_range:
+                end_named_range();
+                break;
+            case XML_named_expression:
+                end_named_expression();
+                break;
             default:
                 ;
         }
@@ -505,39 +560,96 @@ void ods_content_xml_context::start_null_date(const xml_attrs_t& attrs)
     gs->set_origin_date(val.year, val.month, val.day);
 }
 
-void ods_content_xml_context::start_table(const xml_attrs_t& attrs)
+void ods_content_xml_context::start_table(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
 {
-    table_attr_parser parser = for_each(attrs.begin(), attrs.end(), table_attr_parser());
-    const pstring& name = parser.get_name();
-    m_tables.push_back(mp_factory->append_sheet(m_tables.size(), name.get(), name.size()));
+    static const xml_elem_set_t expected = {
+        { NS_odf_office, XML_spreadsheet },
+        { NS_odf_table, XML_dde_link },
+    };
+    xml_element_expected(parent, expected);
 
-    if (get_config().debug)
-        cout << "start table " << name << endl;
+    if (parent == xml_token_pair_t(NS_odf_office, XML_spreadsheet))
+    {
+        table_attr_parser parser = for_each(attrs.begin(), attrs.end(), table_attr_parser());
+        const pstring& name = parser.get_name();
+        m_tables.push_back(mp_factory->append_sheet(m_tables.size(), name.get(), name.size()));
+        m_cur_sheet.sheet = m_tables.back();
+        m_cur_sheet.index = m_tables.size() - 1;
 
-    m_row = m_col = 0;
+        if (get_config().debug)
+            cout << "start table " << name << endl;
+
+        m_row = m_col = 0;
+    }
+    else if (parent == xml_token_pair_t(NS_odf_table, XML_dde_link))
+    {
+        if (get_config().debug)
+            cout << "start table (DDE link)" << endl;
+    }
 }
 
 void ods_content_xml_context::end_table()
 {
-    if (get_config().debug)
-        cout << "end table" << endl;
+    if (m_cur_sheet.sheet)
+    {
+        if (get_config().debug)
+            cout << "end table" << endl;
+
+        m_cur_sheet.reset();
+    }
+}
+
+void ods_content_xml_context::start_named_range(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
+{
+    xml_element_expected(parent, NS_odf_table, XML_named_expressions);
+
+    pick_up_named_range_or_expression(
+        get_session_context(), attrs, NS_odf_table, XML_cell_range_address,
+        ods_session_data::ne_range, m_cur_sheet.index);
+}
+
+void ods_content_xml_context::end_named_range()
+{
+}
+
+void ods_content_xml_context::start_named_expression(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
+{
+    xml_element_expected(parent, NS_odf_table, XML_named_expressions);
+
+    pick_up_named_range_or_expression(
+        get_session_context(), attrs, NS_odf_table, XML_expression,
+        ods_session_data::ne_expression, m_cur_sheet.index);
+}
+
+void ods_content_xml_context::end_named_expression()
+{
 }
 
 void ods_content_xml_context::start_column(const xml_attrs_t& attrs)
 {
-    if (!m_tables.back())
+    if (!m_cur_sheet.sheet)
         return;
 
     spreadsheet::iface::import_sheet_properties* sheet_props =
-        m_tables.back()->get_sheet_properties();
+        m_cur_sheet.sheet->get_sheet_properties();
 
     if (!sheet_props)
         return;
 
-    column_attr_parser func;
-    func = for_each(attrs.begin(), attrs.end(), func);
+    pstring style_name;
 
-    odf_styles_map_type::const_iterator it = m_styles.find(func.get_style_name());
+    std::for_each(attrs.begin(), attrs.end(),
+        [&style_name](const xml_token_attr_t& attr)
+        {
+            if (attr.ns == NS_odf_table)
+            {
+                if (attr.name == XML_style_name)
+                    style_name = attr.value;
+            }
+        }
+    );
+
+    odf_styles_map_type::const_iterator it = m_styles.find(style_name);
     if (it == m_styles.end())
         // Style by this name not found.
         return;
@@ -555,24 +667,49 @@ void ods_content_xml_context::start_row(const xml_attrs_t& attrs)
 {
     m_col = 0;
     m_row_attr = row_attr();
-    row_attr_parser func;
-    func = for_each(attrs.begin(), attrs.end(), func);
-    m_row_attr.number_rows_repeated = func.get_number_rows_repeated();
 
-    if (!m_tables.back())
+    pstring style_name;
+
+    std::for_each(attrs.begin(), attrs.end(),
+        [&](const xml_token_attr_t& attr)
+        {
+            if (attr.ns == NS_odf_table)
+            {
+                switch (attr.name)
+                {
+                    case XML_number_rows_repeated:
+                        m_row_attr.number_rows_repeated = to_long(attr.value);
+                        break;
+                    case XML_style_name:
+                        style_name = attr.value;
+                        break;
+                }
+            }
+        }
+    );
+
+    if (get_config().debug)
+        cout << "row: (style='" << style_name << "')" << endl;
+
+    if (!m_cur_sheet.sheet)
         return;
 
     // Pass row properties to the interface.
     spreadsheet::iface::import_sheet_properties* sheet_props =
-        m_tables.back()->get_sheet_properties();
+        m_cur_sheet.sheet->get_sheet_properties();
 
     if (sheet_props)
     {
-        odf_styles_map_type::const_iterator it = m_styles.find(func.get_style_name());
+        odf_styles_map_type::const_iterator it = m_styles.find(style_name);
         if (it != m_styles.end())
         {
-            const odf_style::row& row_data = *it->second->row_data;
-            sheet_props->set_row_height(m_row, row_data.height.value, row_data.height.unit);
+            const odf_style& style = *it->second;
+            if (style.family == style_family_table_row)
+            {
+                const odf_style::row& row_data = *style.row_data;
+                if (row_data.height_set)
+                    sheet_props->set_row_height(m_row, row_data.height.value, row_data.height.unit);
+            }
         }
     }
 }
@@ -597,8 +734,8 @@ void ods_content_xml_context::start_cell(const xml_attrs_t& attrs)
 void ods_content_xml_context::end_cell()
 {
     name2id_type::const_iterator it = m_cell_format_map.find(m_cell_attr.style_name);
-    if (m_tables.back() && it != m_cell_format_map.end())
-        m_tables.back()->set_format(m_row, m_col, it->second);
+    if (m_cur_sheet.sheet && it != m_cell_format_map.end())
+        m_cur_sheet.sheet->set_format(m_row, m_col, it->second);
 
     push_cell_value();
 
@@ -614,18 +751,18 @@ void ods_content_xml_context::end_cell()
 
 void ods_content_xml_context::push_cell_value()
 {
-    spreadsheet::iface::import_sheet* sheet = m_tables.back();
+    assert(m_cur_sheet.index >= 0); // this is expected to be called only within a valid sheet scope.
 
     bool has_formula = !m_cell_attr.formula.empty();
     if (has_formula)
     {
+        // Store formula cell data for later processing.
         ods_session_data& ods_data =
             static_cast<ods_session_data&>(*get_session_context().mp_data);
-        ods_data.m_formulas.push_back(
-            orcus::make_unique<ods_session_data::formula>(
-                m_tables.size()-1, m_row, m_col, m_cell_attr.formula_grammar, m_cell_attr.formula));
+        ods_data.m_formulas.emplace_back(
+            m_cur_sheet.index, m_row, m_col, m_cell_attr.formula_grammar, m_cell_attr.formula);
 
-        ods_session_data::formula& formula_data = *ods_data.m_formulas.back();
+        ods_session_data::formula& formula_data = ods_data.m_formulas.back();
 
         // Store formula result.
         switch (m_cell_attr.type)
@@ -634,36 +771,36 @@ void ods_content_xml_context::push_cell_value()
             {
                 formula_data.result.type = orcus::ods_session_data::rt_numeric;
                 formula_data.result.numeric_value = m_cell_attr.value;
+                break;
             }
-            break;
             case vt_string:
                 // TODO : pass string result here.  We need to decide whether
                 // to pass a string ID or a raw string.
-            break;
+                break;
             default:
                 ;
         }
         return;
     }
 
-    if (sheet)
+    if (m_cur_sheet.sheet)
     {
         switch (m_cell_attr.type)
         {
             case vt_float:
-                sheet->set_value(m_row, m_col, m_cell_attr.value);
-            break;
+                m_cur_sheet.sheet->set_value(m_row, m_col, m_cell_attr.value);
+                break;
             case vt_string:
                 if (m_has_content)
-                    sheet->set_string(m_row, m_col, m_para_index);
-            break;
+                    m_cur_sheet.sheet->set_string(m_row, m_col, m_para_index);
+                break;
             case vt_date:
             {
                 date_time_t val = to_date_time(m_cell_attr.date_value);
-                sheet->set_date_time(
+                m_cur_sheet.sheet->set_date_time(
                     m_row, m_col, val.year, val.month, val.day, val.hour, val.minute, val.second);
+                break;
             }
-            break;
             default:
                 ;
         }
@@ -675,13 +812,64 @@ void ods_content_xml_context::end_spreadsheet()
     ods_session_data& ods_data =
         static_cast<ods_session_data&>(*get_session_context().mp_data);
 
+    ss::iface::import_reference_resolver* resolver =
+        mp_factory->get_reference_resolver(ss::formula_ref_context_t::named_expression_base);
+
+    if (resolver)
+    {
+        for (const ods_session_data::named_exp& data : ods_data.m_named_exps)
+        {
+            if (get_config().debug)
+            {
+                cout << "named expression: name='" << data.name
+                     << "'; base='" << data.base
+                     << "'; expression='" << data.expression
+                     << "'; sheet-scope=" << data.scope
+                     << endl;
+            }
+
+            ss::src_address_t base = resolver->resolve_address(data.base.data(), data.base.size());
+
+            ss::iface::import_named_expression* named_exp = nullptr;
+
+            if (data.scope >= 0)
+            {
+                // sheet local scope
+                assert(data.scope < ss::sheet_t(m_tables.size()));
+                named_exp = m_tables[data.scope]->get_named_expression();
+            }
+            else
+            {
+                // global scope.
+                named_exp = mp_factory->get_named_expression();
+            }
+
+            if (named_exp)
+            {
+                named_exp->set_base_position(base);
+                switch (data.type)
+                {
+                    case ods_session_data::ne_range:
+                        named_exp->set_named_range(
+                            data.name.data(), data.name.size(), data.expression.data(), data.expression.size());
+                        break;
+                    case ods_session_data::ne_expression:
+                        named_exp->set_named_expression(
+                            data.name.data(), data.name.size(), data.expression.data(), data.expression.size());
+                        break;
+                    default:
+                        ;
+                }
+                named_exp->commit();
+            }
+        }
+    }
+
     // Push all formula cells.  Formula cells needs to be processed after all
     // the sheet data have been imported, else 3D reference would fail to
     // resolve.
-    ods_session_data::formulas_type::iterator it = ods_data.m_formulas.begin(), ite = ods_data.m_formulas.end();
-    for (; it != ite; ++it)
+    for (ods_session_data::formula& data : ods_data.m_formulas)
     {
-        ods_session_data::formula& data = **it;
         if (data.sheet < 0 || static_cast<size_t>(data.sheet) >= m_tables.size())
             // Invalid sheet index.
             continue;
