@@ -12,6 +12,7 @@
 #include "orcus/spreadsheet/import_interface.hpp"
 #include "orcus/exception.hpp"
 #include "orcus/config.hpp"
+#include "orcus/measurement.hpp"
 
 #include "xlsx_types.hpp"
 #include "xlsx_handler.hpp"
@@ -27,6 +28,7 @@
 #include "xlsx_session_data.hpp"
 #include "opc_context.hpp"
 #include "ooxml_global.hpp"
+#include "spreadsheet_iface_util.hpp"
 
 #include <cstdlib>
 #include <iostream>
@@ -38,19 +40,6 @@ using namespace std;
 
 namespace orcus {
 
-namespace {
-
-struct print_sheet_info : unary_function<pair<pstring, const opc_rel_extra*>, void>
-{
-    void operator() (const pair<pstring, const opc_rel_extra*>& v) const
-    {
-        const xlsx_rel_sheet_info* info = static_cast<const xlsx_rel_sheet_info*>(v.second);
-        cout << "sheet name: " << info->name << "  sheet id: " << info->id << "  relationship id: " << v.first << endl;
-    }
-};
-
-}
-
 class xlsx_opc_handler : public opc_reader::part_handler
 {
     orcus_xlsx& m_parent;
@@ -81,6 +70,11 @@ public:
             m_parent.read_styles(dir_path, file_name);
             return true;
         }
+        else if (type == SCH_od_rels_drawing)
+        {
+            m_parent.read_drawing(dir_path, file_name);
+            return true;
+        }
         else if (type == SCH_od_rels_table)
         {
             m_parent.read_table(dir_path, file_name, static_cast<xlsx_rel_table_info*>(data));
@@ -88,12 +82,15 @@ public:
         }
         else if (type == SCH_od_rels_pivot_cache_def)
         {
-            m_parent.read_pivot_cache_def(dir_path, file_name);
+            m_parent.read_pivot_cache_def(
+                dir_path, file_name, static_cast<xlsx_rel_pivot_cache_info*>(data));
             return true;
         }
         else if (type == SCH_od_rels_pivot_cache_rec)
         {
-            m_parent.read_pivot_cache_rec(dir_path, file_name);
+            m_parent.read_pivot_cache_rec(
+                dir_path, file_name,
+                static_cast<const xlsx_rel_pivot_cache_record_info*>(data));
             return true;
         }
         else if (type == SCH_od_rels_pivot_table)
@@ -116,7 +113,7 @@ public:
     }
 };
 
-struct orcus_xlsx_impl
+struct orcus_xlsx::impl
 {
     session_context m_cxt;
     xmlns_repository m_ns_repo;
@@ -124,7 +121,7 @@ struct orcus_xlsx_impl
     xlsx_opc_handler m_opc_handler;
     opc_reader m_opc_reader;
 
-    orcus_xlsx_impl(spreadsheet::iface::import_factory* factory, orcus_xlsx& parent) :
+    impl(spreadsheet::iface::import_factory* factory, orcus_xlsx& parent) :
         m_cxt(new xlsx_session_data),
         mp_factory(factory),
         m_opc_handler(parent),
@@ -132,17 +129,25 @@ struct orcus_xlsx_impl
 };
 
 orcus_xlsx::orcus_xlsx(spreadsheet::iface::import_factory* factory) :
-    mp_impl(new orcus_xlsx_impl(factory, *this))
+    iface::import_filter(format_t::xlsx),
+    mp_impl(orcus::make_unique<impl>(factory, *this))
 {
+    if (!factory)
+        throw std::invalid_argument("factory instance is required.");
+
+    spreadsheet::iface::import_global_settings* gs = factory->get_global_settings();
+    if (gs)
+    {
+        gs->set_origin_date(1899, 12, 30);
+        gs->set_default_formula_grammar(spreadsheet::formula_grammar_t::xlsx);
+    }
+
     mp_impl->m_ns_repo.add_predefined_values(NS_ooxml_all);
     mp_impl->m_ns_repo.add_predefined_values(NS_opc_all);
     mp_impl->m_ns_repo.add_predefined_values(NS_misc_all);
 }
 
-orcus_xlsx::~orcus_xlsx()
-{
-    delete mp_impl;
-}
+orcus_xlsx::~orcus_xlsx() {}
 
 bool orcus_xlsx::detect(const unsigned char* blob, size_t size)
 {
@@ -167,7 +172,7 @@ bool orcus_xlsx::detect(const unsigned char* blob, size_t size)
     if (buf.empty())
         return false;
 
-    config opt;
+    config opt(format_t::xlsx);
     xmlns_repository ns_repo;
     ns_repo.add_predefined_values(NS_opc_all);
     session_context session_cxt;
@@ -230,47 +235,84 @@ void orcus_xlsx::set_formulas_to_doc()
     xlsx_session_data& sdata = static_cast<xlsx_session_data&>(*mp_impl->m_cxt.mp_data);
 
     // Insert shared formulas first.
-    xlsx_session_data::shared_formulas_type::iterator its = sdata.m_shared_formulas.begin(), its_end = sdata.m_shared_formulas.end();
-    for (; its != its_end; ++its)
+    for (auto& p : sdata.m_shared_formulas)
     {
-        xlsx_session_data::shared_formula& sf = **its;
+        xlsx_session_data::shared_formula& sf = *p;
         spreadsheet::iface::import_sheet* sheet = mp_impl->mp_factory->get_sheet(sf.sheet);
         if (!sheet)
             continue;
 
+        spreadsheet::iface::import_formula* formula = sheet->get_formula();
+        if (!formula)
+            continue;
+
+        formula->set_position(sf.row, sf.column);
         if (sf.master)
-        {
-            sheet->set_shared_formula(
-                sf.row, sf.column, orcus::spreadsheet::formula_grammar_t::xlsx_2007, sf.identifier,
-                &sf.formula[0], sf.formula.size(), &sf.range[0], sf.range.size());
-        }
-        else
-        {
-            sheet->set_shared_formula(sf.row, sf.column, sf.identifier);
-        }
+            formula->set_formula(orcus::spreadsheet::formula_grammar_t::xlsx, sf.formula.data(), sf.formula.size());
+        formula->set_shared_formula_index(sf.identifier);
+        formula->commit();
     }
 
     // Insert regular (non-shared) formulas.
-    xlsx_session_data::formulas_type::iterator it = sdata.m_formulas.begin(), it_end = sdata.m_formulas.end();
-    for (; it != it_end; ++it)
+    for (auto& p : sdata.m_formulas)
     {
-        xlsx_session_data::formula& f = **it;
+        xlsx_session_data::formula& f = *p;
         spreadsheet::iface::import_sheet* sheet = mp_impl->mp_factory->get_sheet(f.sheet);
         if (!sheet)
             continue;
 
-        if (f.array)
+        spreadsheet::iface::import_formula* formula = sheet->get_formula();
+        if (!formula)
+            continue;
+
+        formula->set_position(f.ref.row, f.ref.column);
+        formula->set_formula(orcus::spreadsheet::formula_grammar_t::xlsx, f.exp.data(), f.exp.size());
+        formula->commit();
+    }
+
+    // Insert array formulas.
+    for (auto& p : sdata.m_array_formulas)
+    {
+        xlsx_session_data::array_formula& af = *p;
+        spreadsheet::iface::import_sheet* sheet = mp_impl->mp_factory->get_sheet(af.sheet);
+        if (!sheet)
+            continue;
+
+        spreadsheet::iface::import_array_formula* xaf = sheet->get_array_formula();
+        push_array_formula(xaf, af.ref, af.exp, spreadsheet::formula_grammar_t::xlsx, *af.results);
+    }
+}
+
+namespace {
+
+size_t get_schema_rank(const schema_t sch)
+{
+    using map_type = std::unordered_map <schema_t, size_t>;
+
+    static const schema_t schema_rank[] = {
+        SCH_od_rels_shared_strings,
+        SCH_od_rels_pivot_cache_def,
+        SCH_od_rels_worksheet,
+        nullptr
+    };
+
+    static map_type rank_map;
+
+    if (rank_map.empty())
+    {
+        // initialize it.
+        size_t rank = 0;
+        for (const schema_t* p = schema_rank; *p; ++rank, ++p)
         {
-            sheet->set_array_formula(
-                f.row, f.column, spreadsheet::formula_grammar_t::xlsx_2007, &f.exp[0],
-                f.exp.size(), &f.range[0], f.range.size());
-        }
-        else
-        {
-            sheet->set_formula(
-                f.row, f.column, orcus::spreadsheet::formula_grammar_t::xlsx_2007, &f.exp[0], f.exp.size());
+            rank_map.insert(
+                map_type::value_type(*p, rank));
         }
     }
+
+    auto it = rank_map.find(sch);
+    return it == rank_map.end() ? numeric_limits<size_t>::max() : it->second;
+}
+
 }
 
 void orcus_xlsx::read_workbook(const string& dir_path, const string& file_name)
@@ -287,7 +329,7 @@ void orcus_xlsx::read_workbook(const string& dir_path, const string& file_name)
         return;
 
     auto handler = orcus::make_unique<xml_simple_stream_handler>(
-        new xlsx_workbook_context(mp_impl->m_cxt, ooxml_tokens));
+        new xlsx_workbook_context(mp_impl->m_cxt, ooxml_tokens, *mp_impl->mp_factory));
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -298,13 +340,59 @@ void orcus_xlsx::read_workbook(const string& dir_path, const string& file_name)
     // Get sheet info from the context instance.
     xlsx_workbook_context& context =
         static_cast<xlsx_workbook_context&>(handler->get_context());
-    opc_rel_extras_t sheet_data;
-    context.pop_sheet_info(sheet_data);
+    opc_rel_extras_t workbook_data;
+    context.pop_workbook_info(workbook_data);
     if (get_config().debug)
-        for_each(sheet_data.data.begin(), sheet_data.data.end(), print_sheet_info());
+    {
+        for_each(workbook_data.data.begin(), workbook_data.data.end(),
+            [](const opc_rel_extras_t::map_type::value_type& v)
+            {
+                const xlsx_rel_sheet_info* info =
+                    dynamic_cast<const xlsx_rel_sheet_info*>(v.second.get());
+
+                if (info)
+                {
+                    cout << "relationship id: " << v.first << "; sheet name: " << info->name << "; sheet id: " << info->id << endl;
+                }
+
+                const xlsx_rel_pivot_cache_info* info_pc =
+                    dynamic_cast<const xlsx_rel_pivot_cache_info*>(v.second.get());
+
+                if (info_pc)
+                {
+                    cout << "relationship id: " << v.first << "; pivot cache id: " << info_pc->id << endl;
+                }
+            }
+        );
+    }
 
     handler.reset();
-    mp_impl->m_opc_reader.check_relation_part(file_name, &sheet_data);
+
+    // Re-order the relation items so that shared strings get imported first,
+    // pivot caches get imported before the sheets and so on.
+
+    opc_reader::sort_compare_type sort_func =
+        [](const opc_rel_t& left, const opc_rel_t& right)
+        {
+            size_t rank_left = get_schema_rank(left.type), rank_right = get_schema_rank(right.type);
+            if (rank_left != rank_right)
+                return rank_left < rank_right;
+
+            pstring rid1 = left.rid, rid2 = right.rid;
+
+            if (rid1.size() > 1 && rid2.size() > 1)
+            {
+                // numerical comparison of relation ID's.
+                rid1 = pstring(&rid1[1], rid1.size()-1); // remove the 'r' prefix.
+                rid2 = pstring(&rid2[1], rid2.size()-1); // remove the 'r' prefix.
+                return to_long(rid1) < to_long(rid2);
+            }
+
+            // textural comparison of relation ID's.
+            return left.rid < right.rid;
+        };
+
+    mp_impl->m_opc_reader.check_relation_part(file_name, &workbook_data, &sort_func);
 }
 
 void orcus_xlsx::read_sheet(const string& dir_path, const string& file_name, xlsx_rel_sheet_info* data)
@@ -333,16 +421,26 @@ void orcus_xlsx::read_sheet(const string& dir_path, const string& file_name, xls
         cout << "  sheet name: " << data->name << "  sheet ID: " << data->id << endl;
     }
 
-    spreadsheet::iface::import_sheet* sheet = mp_impl->mp_factory->append_sheet(data->name.get(), data->name.size());
+    spreadsheet::iface::import_sheet* sheet = mp_impl->mp_factory->get_sheet(data->name.get(), data->name.size());
     if (!sheet)
-        throw general_error("orcus_xlsx::read_sheet: failed to append sheet.");
+    {
+        std::ostringstream os;
+        os << "orcus_xlsx::read_sheet: ";
+        os << "sheet named '" << data->name << "' doesn't exist.";
+        throw general_error(os.str());
+    }
+
+    spreadsheet::iface::import_reference_resolver* resolver =
+        mp_impl->mp_factory->get_reference_resolver(spreadsheet::formula_ref_context_t::global);
+    if (!resolver)
+        throw general_error("orcus_xlsx::read_sheet: reference resolver interface is not available.");
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
 
     auto handler = orcus::make_unique<xlsx_sheet_xml_handler>(
-        mp_impl->m_cxt, ooxml_tokens, data->id-1, sheet);
+        mp_impl->m_cxt, ooxml_tokens, data->id-1, *resolver, *sheet);
 
     parser.set_handler(handler.get());
     parser.parse();
@@ -424,6 +522,13 @@ void orcus_xlsx::read_table(const std::string& dir_path, const std::string& file
         // Client code doesn't support tables. No point going further.
         return;
 
+    spreadsheet::iface::import_reference_resolver* resolver =
+        mp_impl->mp_factory->get_reference_resolver(spreadsheet::formula_ref_context_t::global);
+
+    if (!resolver)
+        // This client doesn't support reference resolver, but is required.
+        return;
+
     string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
@@ -441,7 +546,8 @@ void orcus_xlsx::read_table(const std::string& dir_path, const std::string& file
     if (buffer.empty())
         return;
 
-    auto handler = orcus::make_unique<xlsx_table_xml_handler>(mp_impl->m_cxt, ooxml_tokens, *table);
+    auto handler = orcus::make_unique<xlsx_table_xml_handler>(
+        mp_impl->m_cxt, ooxml_tokens, *table, *resolver);
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -452,13 +558,26 @@ void orcus_xlsx::read_table(const std::string& dir_path, const std::string& file
     handler.reset();
 }
 
-void orcus_xlsx::read_pivot_cache_def(const std::string& dir_path, const std::string& file_name)
+void orcus_xlsx::read_pivot_cache_def(
+    const std::string& dir_path, const std::string& file_name,
+    const xlsx_rel_pivot_cache_info* data)
 {
+    if (!data)
+    {
+        if (get_config().debug)
+        {
+            cout << "---" << endl;
+            cout << "required pivot cache relation info was not present." << endl;
+        }
+        return;
+    }
+
     string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
         cout << "---" << endl;
-        cout << "read_pivot_cache_def: file path = " << filepath << endl;
+        cout << "read_pivot_cache_def: file path = " << filepath
+            << "; cache id = " << data->id << endl;
     }
 
     vector<unsigned char> buffer;
@@ -471,7 +590,15 @@ void orcus_xlsx::read_pivot_cache_def(const std::string& dir_path, const std::st
     if (buffer.empty())
         return;
 
-    auto handler = orcus::make_unique<xlsx_pivot_cache_def_xml_handler>(mp_impl->m_cxt, ooxml_tokens);
+    spreadsheet::iface::import_pivot_cache_definition* pcache =
+        mp_impl->mp_factory->create_pivot_cache_definition(data->id);
+
+    if (!pcache)
+        // failed to create a cache instance for whatever reason.
+        return;
+
+    auto handler = orcus::make_unique<xlsx_pivot_cache_def_xml_handler>(
+        mp_impl->m_cxt, ooxml_tokens, *pcache, data->id);
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -479,17 +606,31 @@ void orcus_xlsx::read_pivot_cache_def(const std::string& dir_path, const std::st
     parser.set_handler(handler.get());
     parser.parse();
 
+    opc_rel_extras_t pcache_info = handler->pop_rel_extras();
+
     handler.reset();
-    mp_impl->m_opc_reader.check_relation_part(file_name, NULL);
+    mp_impl->m_opc_reader.check_relation_part(file_name, &pcache_info);
 }
 
-void orcus_xlsx::read_pivot_cache_rec(const std::string& dir_path, const std::string& file_name)
+void orcus_xlsx::read_pivot_cache_rec(
+    const std::string& dir_path, const std::string& file_name,
+    const xlsx_rel_pivot_cache_record_info* data)
 {
+    if (!data)
+    {
+        if (get_config().debug)
+        {
+            cout << "---" << endl;
+            cout << "required pivot cache record relation info was not present." << endl;
+        }
+        return;
+    }
+
     string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
         cout << "---" << endl;
-        cout << "read_pivot_cache_rec: file path = " << filepath << endl;
+        cout << "read_pivot_cache_rec: file path = " << filepath << "; cache id = " << data->id << endl;
     }
 
     vector<unsigned char> buffer;
@@ -502,7 +643,14 @@ void orcus_xlsx::read_pivot_cache_rec(const std::string& dir_path, const std::st
     if (buffer.empty())
         return;
 
-    auto handler = orcus::make_unique<xlsx_pivot_cache_rec_xml_handler>(mp_impl->m_cxt, ooxml_tokens);
+    spreadsheet::iface::import_pivot_cache_records* pcache_records =
+        mp_impl->mp_factory->create_pivot_cache_records(data->id);
+
+    if (!pcache_records)
+        return;
+
+    auto handler = orcus::make_unique<xlsx_pivot_cache_rec_xml_handler>(
+        mp_impl->m_cxt, ooxml_tokens, *pcache_records);
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -541,7 +689,7 @@ void orcus_xlsx::read_pivot_table(const std::string& dir_path, const std::string
     parser.parse();
 
     handler.reset();
-    mp_impl->m_opc_reader.check_relation_part(file_name, NULL);
+    mp_impl->m_opc_reader.check_relation_part(file_name, nullptr);
 }
 
 void orcus_xlsx::read_rev_headers(const std::string& dir_path, const std::string& file_name)
@@ -574,7 +722,7 @@ void orcus_xlsx::read_rev_headers(const std::string& dir_path, const std::string
     parser.parse();
 
     handler.reset();
-    mp_impl->m_opc_reader.check_relation_part(file_name, NULL);
+    mp_impl->m_opc_reader.check_relation_part(file_name, nullptr);
 }
 
 void orcus_xlsx::read_rev_log(const std::string& dir_path, const std::string& file_name)
@@ -609,5 +757,36 @@ void orcus_xlsx::read_rev_log(const std::string& dir_path, const std::string& fi
     handler.reset();
 }
 
+void orcus_xlsx::read_drawing(const std::string& dir_path, const std::string& file_name)
+{
+    string filepath = resolve_file_path(dir_path, file_name);
+    if (get_config().debug)
+    {
+        cout << "---" << endl;
+        cout << "read_drawing: file path = " << filepath << endl;
+    }
+
+    vector<unsigned char> buffer;
+    if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
+    {
+        cerr << "failed to open zip stream: " << filepath << endl;
+        return;
+    }
+
+    if (buffer.empty())
+        return;
+
+    auto handler = orcus::make_unique<xlsx_drawing_xml_handler>(
+        mp_impl->m_cxt, ooxml_tokens);
+
+    xml_stream_parser parser(
+        get_config(), mp_impl->m_ns_repo, ooxml_tokens,
+        reinterpret_cast<const char*>(&buffer[0]), buffer.size());
+    parser.set_handler(handler.get());
+    parser.parse();
+
+    handler.reset();
+}
+
 }
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
