@@ -246,6 +246,24 @@ const map_type& get()
 
 } // namespace named_color
 
+namespace formula_error_policy {
+
+using map_type = mdds::sorted_string_map<formula_error_policy_t>;
+
+// Keys must be sorted.
+const std::vector<map_type::entry> entries = {
+    { ORCUS_ASCII("fail"), formula_error_policy_t::fail },
+    { ORCUS_ASCII("skip"), formula_error_policy_t::skip },
+};
+
+const map_type& get()
+{
+    static map_type mt(entries.data(), entries.size(), formula_error_policy_t::unknown);
+    return mt;
+}
+
+}
+
 } // anonymous namespace
 
 color_rgb_t::color_rgb_t() : red(0), green(0), blue(0) {}
@@ -277,6 +295,24 @@ color_rgb_t::color_rgb_t(color_rgb_t&& other) :
     other.blue = 0;
 }
 
+address_t to_rc_address(const src_address_t& r)
+{
+    address_t ret;
+    ret.row = r.row;
+    ret.column = r.column;
+    return ret;
+}
+
+range_t to_rc_range(const src_range_t& r)
+{
+    range_t ret;
+    ret.first.row = r.first.row;
+    ret.first.column = r.first.column;
+    ret.last.row = r.last.row;
+    ret.last.column = r.last.column;
+    return ret;
+}
+
 bool operator== (const address_t& left, const address_t& right)
 {
     return left.column == right.column && left.row == right.row;
@@ -287,6 +323,16 @@ bool operator!= (const address_t& left, const address_t& right)
     return !operator== (left, right);
 }
 
+bool operator== (const src_address_t& left, const src_address_t& right)
+{
+    return left.column == right.column && left.row == right.row;
+}
+
+bool operator!= (const src_address_t& left, const src_address_t& right)
+{
+    return !operator== (left, right);
+}
+
 bool operator== (const range_t& left, const range_t& right)
 {
     return left.first == right.first && left.last == right.last;
@@ -297,6 +343,16 @@ bool operator!= (const range_t& left, const range_t& right)
     return !operator== (left, right);
 }
 
+bool operator== (const src_range_t& left, const src_range_t& right)
+{
+    return left.first == right.first && left.last == right.last;
+}
+
+bool operator!= (const src_range_t& left, const src_range_t& right)
+{
+    return !operator== (left, right);
+}
+
 bool operator< (const range_t& left, const range_t& right)
 {
     if (left.first.row != right.first.row)
@@ -351,6 +407,12 @@ std::ostream& operator<< (std::ostream& os, const address_t& v)
     return os;
 }
 
+std::ostream& operator<< (std::ostream& os, const src_address_t& v)
+{
+    os << "(sheet=" << v.sheet << ",column=" << v.column << ",row=" << v.row << ")";
+    return os;
+}
+
 std::ostream& operator<< (std::ostream& os, const range_t& v)
 {
     os << v.first << "-" << v.last;
@@ -455,6 +517,11 @@ color_rgb_t to_color_rgb_from_name(const char* p, size_t n)
     return named_colors::get().find(p, n);
 }
 
+formula_error_policy_t to_formula_error_policy(const char* p, size_t n)
+{
+    return formula_error_policy::get().find(p, n);
+}
+
 std::ostream& operator<< (std::ostream& os, error_value_t ev)
 {
     switch (ev)
