@@ -18,9 +18,37 @@
 #include <iostream>
 
 using namespace std;
+namespace ss = orcus::spreadsheet;
 
 namespace orcus {
 
+namespace {
+
+spreadsheet::color_rgb_t to_rgb(const pstring& s)
+{
+    if (!s.empty() && s[0] == '#')
+        return spreadsheet::to_color_rgb(s.data(), s.size());
+    else
+    {
+        // This may be a color name.  Lower-case it before sending it to the
+        // function.
+        std::string s_lower(s.size(), '\0');
+        const char* p = s.data();
+        std::transform(p, p + s.size(), s_lower.begin(),
+            [](char c) -> char
+            {
+                if ('A' <= c && c <= 'Z')
+                    c += 'a' - 'A';
+                return c;
+            }
+        );
+
+        return spreadsheet::to_color_rgb_from_name(s_lower.data(), s_lower.size());
+    }
+}
+
+}
+
 void xls_xml_data_context::format_type::merge(const format_type& fmt)
 {
     if (fmt.bold)
@@ -117,8 +145,7 @@ void xls_xml_data_context::start_element(xmlns_id_t ns, xml_token_t name, const:
                     switch (attr.name)
                     {
                         case XML_Color:
-                            fmt.color = spreadsheet::to_color_rgb(
-                                attr.value.data(), attr.value.size());
+                            fmt.color = to_rgb(attr.value);
                             break;
                         default:
                             ;
@@ -233,29 +260,27 @@ void xls_xml_data_context::start_element_data(
     m_cell_string.clear();
     m_cell_datetime = date_time_t();
 
-    std::for_each(attrs.begin(), attrs.end(),
-        [&](const xml_token_attr_t& attr)
-        {
-            if (attr.ns != NS_xls_xml_ss)
-                return;
+    for (const xml_token_attr_t& attr : attrs)
+    {
+        if (attr.ns != NS_xls_xml_ss)
+            continue;
 
-            switch (attr.name)
+        switch (attr.name)
+        {
+            case XML_Type:
             {
-                case XML_Type:
-                {
-                    if (attr.value == "String")
-                        m_cell_type = ct_string;
-                    else if (attr.value == "Number")
-                        m_cell_type = ct_number;
-                    else if (attr.value == "DateTime")
-                        m_cell_type = ct_datetime;
-                }
+                if (attr.value == "String")
+                    m_cell_type = ct_string;
+                else if (attr.value == "Number")
+                    m_cell_type = ct_number;
+                else if (attr.value == "DateTime")
+                    m_cell_type = ct_datetime;
                 break;
-                default:
-                    ;
             }
+            default:
+                ;
         }
-    );
+    }
 }
 
 void xls_xml_data_context::end_element_data()
@@ -265,7 +290,7 @@ void xls_xml_data_context::end_element_data()
     if (!formula.empty())
     {
         if (m_parent_cxt.is_array_formula())
-            push_array_formula_parent_cell(formula);
+            store_array_formula_parent_cell(formula);
         else
             push_formula_cell(formula);
         m_cell_type = ct_unknown;
@@ -423,29 +448,20 @@ void xls_xml_data_context::push_array_result(
 
 void xls_xml_data_context::push_formula_cell(const pstring& formula)
 {
-    spreadsheet::iface::import_sheet* sheet = m_parent_cxt.get_import_sheet();
-    spreadsheet::address_t pos = m_parent_cxt.get_current_pos();
-    spreadsheet::iface::import_formula* xformula = sheet->get_formula();
-
-    if (xformula)
+    switch (m_cell_type)
     {
-        xformula->set_position(pos.row, pos.column);
-        xformula->set_formula(spreadsheet::formula_grammar_t::xls_xml, formula.data(), formula.size());
-
-        switch (m_cell_type)
+        case ct_number:
+            m_parent_cxt.store_cell_formula(formula, m_cell_value);
+            break;
+        default:
         {
-            case ct_number:
-                xformula->set_result_value(m_cell_value);
-                break;
-            default:
-                ;
+            formula_result res;
+            m_parent_cxt.store_cell_formula(formula, res);
         }
-
-        xformula->commit();
     }
 }
 
-void xls_xml_data_context::push_array_formula_parent_cell(const pstring& formula)
+void xls_xml_data_context::store_array_formula_parent_cell(const pstring& formula)
 {
     spreadsheet::address_t pos = m_parent_cxt.get_current_pos();
     spreadsheet::range_t range = m_parent_cxt.get_array_range();
@@ -491,28 +507,6 @@ void xls_xml_data_context::update_current_format()
 
 namespace {
 
-class sheet_attr_parser : public unary_function<xml_token_attr_t, void>
-{
-    pstring m_name;
-public:
-    void operator() (const xml_token_attr_t& attr)
-    {
-        if (attr.ns == NS_xls_xml_ss)
-        {
-            switch (attr.name)
-            {
-                case XML_Name:
-                    m_name = attr.value;
-                break;
-                default:
-                    ;
-            }
-        }
-    }
-
-    pstring get_name() const { return m_name; }
-};
-
 namespace border_dir {
 
 typedef mdds::sorted_string_map<spreadsheet::border_direction_t> map_type;
@@ -604,12 +598,46 @@ const map_type& get()
 
 }
 
+namespace num_format {
+
+typedef mdds::sorted_string_map<pstring> map_type;
+
+// Keys must be sorted.
+const std::vector<map_type::entry> entries =
+{
+    { ORCUS_ASCII("Currency"), "$#,##0.00_);[Red]($#,##0.00)" },
+    { ORCUS_ASCII("Euro Currency"), "[$\xe2\x82\xac-x-euro2] #,##0.00_);[Red]([$\xe2\x82\xac-x-euro2] #,##0.00)" },
+    { ORCUS_ASCII("Fixed"), "0.00" },
+    { ORCUS_ASCII("General Date"), "m/d/yyyy h:mm" },
+    { ORCUS_ASCII("General Number"), "General" },
+    { ORCUS_ASCII("Long Date"), "d-mmm-yy" },
+    { ORCUS_ASCII("Long Time"), "h:mm:ss AM/PM" },
+    { ORCUS_ASCII("Medium Date"), "d-mmm-yy" },
+    { ORCUS_ASCII("Medium Time"), "h:mm AM/PM" },
+    { ORCUS_ASCII("On/Off"), "\"On\";\"On\";\"Off\"" },
+    { ORCUS_ASCII("Percent"), "0.00%" },
+    { ORCUS_ASCII("Scientific"), "0.00E+00" },
+    { ORCUS_ASCII("Short Date"), "m/d/yyyy" },
+    { ORCUS_ASCII("Short Time"), "h:mm" },
+    { ORCUS_ASCII("Standard"), "#,##0.00" },
+    { ORCUS_ASCII("True/False"), "\"True\";\"True\";\"False\"" },
+    { ORCUS_ASCII("Yes/No"), "\"Yes\";\"Yes\";\"No\"" },
+};
+
+const map_type& get()
+{
+    static map_type mt(entries.data(), entries.size(), pstring());
+    return mt;
 }
 
+} // namespace num_format
+
+} // anonymous namespace
+
 xls_xml_context::array_formula_type::array_formula_type(
-    const spreadsheet::range_t& range, const pstring& formula) :
-    formula(formula),
-    results(range.last.row-range.first.row+1, range.last.column-range.first.column+1) {}
+    const spreadsheet::range_t& _range, const pstring& _formula) :
+    formula(_formula),
+    results(_range.last.row-_range.first.row+1, _range.last.column-_range.first.column+1) {}
 
 xls_xml_context::named_exp::named_exp(const pstring& _name, const pstring& _expression, spreadsheet::sheet_t _scope) :
     name(_name), expression(_expression), scope(_scope) {}
@@ -673,6 +701,17 @@ spreadsheet::address_t xls_xml_context::split_pane::get_top_left_cell() const
     return pos;
 }
 
+xls_xml_context::table_properties::table_properties()
+{
+    reset();
+}
+
+void xls_xml_context::table_properties::reset()
+{
+    pos.row = 0;
+    pos.column = 0;
+}
+
 xls_xml_context::xls_xml_context(session_context& session_cxt, const tokens& tokens, spreadsheet::iface::import_factory* factory) :
     xml_context_base(session_cxt, tokens),
     mp_factory(factory),
@@ -682,7 +721,6 @@ xls_xml_context::xls_xml_context(session_context& session_cxt, const tokens& tok
     m_cur_row(0), m_cur_col(0),
     m_cur_prop_col(0),
     m_cur_merge_down(0), m_cur_merge_across(0),
-    m_active_pane(spreadsheet::sheet_pane_t::unspecified),
     m_cc_data(session_cxt, tokens, *this)
 {
     m_cur_array_range.first.column = -1;
@@ -754,26 +792,11 @@ void xls_xml_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_a
                 break;
             case XML_Worksheet:
             {
-                xml_element_expected(parent, NS_xls_xml_ss, XML_Workbook);
-
-                ++m_cur_sheet;
-                pstring sheet_name = for_each(attrs.begin(), attrs.end(), sheet_attr_parser()).get_name();
-                mp_cur_sheet = mp_factory->append_sheet(m_cur_sheet, sheet_name.get(), sheet_name.size());
-                spreadsheet::iface::import_named_expression* sheet_named_exp = nullptr;
-                if (mp_cur_sheet)
-                {
-                    mp_sheet_props = mp_cur_sheet->get_sheet_properties();
-                    sheet_named_exp = mp_cur_sheet->get_named_expression();
-                }
-
-                m_sheet_named_exps.push_back(sheet_named_exp);
-
-                m_cur_row = 0;
-                m_cur_col = 0;
+                start_element_worksheet(parent, attrs);
                 break;
             }
             case XML_Table:
-                xml_element_expected(parent, NS_xls_xml_ss, XML_Worksheet);
+                start_element_table(parent, attrs);
                 break;
             case XML_Row:
                 start_element_row(parent, attrs);
@@ -797,7 +820,7 @@ void xls_xml_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_a
             {
                 xml_element_expected(parent, NS_xls_xml_ss, XML_Names);
 
-                pstring name, exp;
+                pstring name_s, exp;
 
                 for (const xml_token_attr_t& attr : attrs)
                 {
@@ -807,7 +830,7 @@ void xls_xml_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_a
                     switch (attr.name)
                     {
                         case XML_Name:
-                            name = intern(attr);
+                            name_s = intern(attr);
                             break;
                         case XML_RefersTo:
                         {
@@ -823,12 +846,12 @@ void xls_xml_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_a
                     }
                 }
 
-                if (!name.empty() && !exp.empty())
+                if (!name_s.empty() && !exp.empty())
                 {
                     if (m_cur_sheet >= 0)
-                        m_named_exps_sheet.emplace_back(name, exp, m_cur_sheet);
+                        m_named_exps_sheet.emplace_back(name_s, exp, m_cur_sheet);
                     else
-                        m_named_exps_global.emplace_back(name, exp, -1);
+                        m_named_exps_global.emplace_back(name_s, exp, -1);
                 }
 
                 break;
@@ -874,6 +897,9 @@ void xls_xml_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_a
             case XML_Border:
                 start_element_border(parent, attrs);
                 break;
+            case XML_NumberFormat:
+                start_element_number_format(parent, attrs);
+                break;
             case XML_Font:
             {
                 xml_element_expected(parent, NS_xls_xml_ss, XML_Style);
@@ -897,8 +923,7 @@ void xls_xml_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_a
                         }
                         case XML_Color:
                         {
-                            m_current_style->font.color =
-                                spreadsheet::to_color_rgb(attr.value.data(), attr.value.size());
+                            m_current_style->font.color = to_rgb(attr.value);
                             break;
                         }
                         default:
@@ -920,8 +945,7 @@ void xls_xml_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_a
                     {
                         case XML_Color:
                         {
-                            m_current_style->fill.color =
-                                spreadsheet::to_color_rgb(attr.value.data(), attr.value.size());
+                            m_current_style->fill.color = to_rgb(attr.value);
                             break;
                         }
                         case XML_Pattern:
@@ -1061,6 +1085,9 @@ bool xls_xml_context::end_element(xmlns_id_t ns, xml_token_t name)
             case XML_Border:
                 end_element_border();
                 break;
+            case XML_NumberFormat:
+                end_element_number_format();
+                break;
             case XML_Row:
                 end_element_row();
                 break;
@@ -1077,7 +1104,7 @@ bool xls_xml_context::end_element(xmlns_id_t ns, xml_token_t name)
                 end_element_workbook();
                 break;
             case XML_Worksheet:
-                mp_cur_sheet = nullptr;
+                end_element_worksheet();
                 break;
             case XML_Style:
             {
@@ -1177,10 +1204,10 @@ void xls_xml_context::characters(const pstring& str, bool /*transient*/)
             case XML_RangeSelection:
             {
                 spreadsheet::iface::import_reference_resolver* resolver =
-                    mp_factory->get_reference_resolver();
+                    mp_factory->get_reference_resolver(spreadsheet::formula_ref_context_t::global);
 
                 if (resolver)
-                    m_cursor_selection.range = resolver->resolve_range(str.data(), str.size());
+                    m_cursor_selection.range = to_rc_range(resolver->resolve_range(str.data(), str.size()));
 
                 break;
             }
@@ -1229,7 +1256,7 @@ void xls_xml_context::start_element_border(const xml_token_pair_t& parent, const
             }
             case XML_Color:
             {
-                color = spreadsheet::to_color_rgb(attr.value.data(), attr.value.size());
+                color = to_rgb(attr.value);
                 break;
             }
             default:
@@ -1286,6 +1313,30 @@ void xls_xml_context::start_element_border(const xml_token_pair_t& parent, const
     }
 }
 
+void xls_xml_context::start_element_number_format(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
+{
+    xml_element_expected(parent, NS_xls_xml_ss, XML_Style);
+    m_current_style->number_format.clear();
+
+    for (const xml_token_attr_t& attr : attrs)
+    {
+        if (attr.ns != NS_xls_xml_ss)
+            continue;
+
+        switch (attr.name)
+        {
+            case XML_Format:
+            {
+                pstring code = num_format::get().find(attr.value.data(), attr.value.size());
+                m_current_style->number_format = code.empty() ? intern(attr) : code;
+                break;
+            }
+            default:
+                ;
+        }
+    }
+}
+
 void xls_xml_context::start_element_cell(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
 {
     xml_element_expected(parent, NS_xls_xml_ss, XML_Row);
@@ -1334,9 +1385,10 @@ void xls_xml_context::start_element_cell(const xml_token_pair_t& parent, const x
                 break;
             case XML_ArrayRange:
             {
-                spreadsheet::iface::import_reference_resolver* resolver = mp_factory->get_reference_resolver();
+                spreadsheet::iface::import_reference_resolver* resolver =
+                    mp_factory->get_reference_resolver(spreadsheet::formula_ref_context_t::global);
                 if (resolver)
-                    m_cur_array_range = resolver->resolve_range(attr.value.data(), attr.value.size());
+                    m_cur_array_range = to_rc_range(resolver->resolve_range(attr.value.data(), attr.value.size()));
 
                 break;
             }
@@ -1351,7 +1403,7 @@ void xls_xml_context::start_element_cell(const xml_token_pair_t& parent, const x
     if (col_index > 0)
     {
         // 1-based column index. Convert it to a 0-based one.
-        m_cur_col = col_index - 1;
+        m_cur_col = m_table_props.pos.column + col_index - 1;
     }
 }
 
@@ -1409,7 +1461,7 @@ void xls_xml_context::start_element_column(const xml_token_pair_t& parent, const
 void xls_xml_context::start_element_row(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
 {
     xml_element_expected(parent, NS_xls_xml_ss, XML_Table);
-    m_cur_col = 0;
+    m_cur_col = m_table_props.pos.column;
     spreadsheet::row_t row_index = -1;
     bool has_height = false;
     bool hidden = false;
@@ -1456,6 +1508,86 @@ void xls_xml_context::start_element_row(const xml_token_pair_t& parent, const xm
     }
 }
 
+void xls_xml_context::start_element_table(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
+{
+    xml_element_expected(parent, NS_xls_xml_ss, XML_Worksheet);
+
+    spreadsheet::row_t row_index = -1;
+    spreadsheet::col_t col_index = -1;
+
+    for (const xml_token_attr_t& attr : attrs)
+    {
+        if (attr.value.empty())
+            return;
+
+        if (attr.ns == NS_xls_xml_ss)
+        {
+            switch (attr.name)
+            {
+                case XML_TopCell:
+                    col_index = to_long(attr.value);
+                    break;
+                case XML_LeftCell:
+                    row_index = to_long(attr.value);
+                    break;
+                default:
+                    ;
+            }
+        }
+    }
+
+    // Convert 1-based indices to 0-based.
+
+    if (row_index > 0)
+    {
+        m_table_props.pos.row = row_index - 1;
+        m_cur_row = m_table_props.pos.row;
+    }
+
+    if (col_index > 0)
+        m_table_props.pos.column = col_index - 1;
+}
+
+void xls_xml_context::start_element_worksheet(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
+{
+    xml_element_expected(parent, NS_xls_xml_ss, XML_Workbook);
+
+    ++m_cur_sheet;
+    pstring sheet_name;
+    m_cell_formulas.emplace_back();
+
+    for (const xml_token_attr_t& attr : attrs)
+    {
+        if (attr.ns == NS_xls_xml_ss)
+        {
+            switch (attr.name)
+            {
+                case XML_Name:
+                    sheet_name = attr.value;
+                    break;
+                default:
+                    ;
+            }
+        }
+    }
+
+    mp_cur_sheet = mp_factory->append_sheet(m_cur_sheet, sheet_name.data(), sheet_name.size());
+    spreadsheet::iface::import_named_expression* sheet_named_exp = nullptr;
+    if (mp_cur_sheet)
+    {
+        mp_sheet_props = mp_cur_sheet->get_sheet_properties();
+        sheet_named_exp = mp_cur_sheet->get_named_expression();
+    }
+
+    m_sheet_named_exps.push_back(sheet_named_exp);
+
+    m_cur_row = 0;
+    m_cur_col = 0;
+
+    if (get_config().debug)
+        cout << "worksheet: name: '" << sheet_name << "'" << endl;
+}
+
 void xls_xml_context::end_element_borders()
 {
 }
@@ -1464,6 +1596,10 @@ void xls_xml_context::end_element_border()
 {
 }
 
+void xls_xml_context::end_element_number_format()
+{
+}
+
 void xls_xml_context::end_element_cell()
 {
     if (mp_sheet_props && (m_cur_merge_across > 0 || m_cur_merge_down > 0))
@@ -1490,16 +1626,7 @@ void xls_xml_context::end_element_cell()
     if (mp_cur_sheet && !m_cur_cell_formula.empty())
     {
         // Likely a Cell element without a child Data element.
-        spreadsheet::iface::import_formula* xformula = mp_cur_sheet->get_formula();
-
-        if (xformula)
-        {
-            xformula->set_position(m_cur_row, m_cur_col);
-            xformula->set_formula(
-                spreadsheet::formula_grammar_t::xls_xml,
-                m_cur_cell_formula.data(), m_cur_cell_formula.size());
-            xformula->commit();
-        }
+        store_cell_formula(m_cur_cell_formula, formula_result());
     }
 
     m_cur_cell_formula.clear();
@@ -1522,10 +1649,19 @@ void xls_xml_context::end_element_table()
 {
     push_all_array_formulas();
     m_array_formulas.clear();
+    m_table_props.reset();
+}
+
+void xls_xml_context::end_element_worksheet()
+{
+    mp_cur_sheet = nullptr;
 }
 
 void xls_xml_context::end_element_workbook()
 {
+    if (!mp_factory)
+        return;
+
     spreadsheet::iface::import_named_expression* ne_global = mp_factory->get_named_expression();
     if (ne_global)
     {
@@ -1533,8 +1669,9 @@ void xls_xml_context::end_element_workbook()
 
         for (const named_exp& ne : m_named_exps_global)
         {
-            ne_global->define_name(
+            ne_global->set_named_expression(
                 ne.name.data(), ne.name.size(), ne.expression.data(), ne.expression.size());
+            ne_global->commit();
         }
     }
 
@@ -1547,8 +1684,43 @@ void xls_xml_context::end_element_workbook()
             p = m_sheet_named_exps[ne.scope]; // it may be nullptr.
 
         if (p)
-            p->define_name(
+        {
+            p->set_named_expression(
                 ne.name.data(), ne.name.size(), ne.expression.data(), ne.expression.size());
+            p->commit();
+        }
+    }
+
+    // push all cell formulas
+    for (size_t sheet_pos = 0; sheet_pos < m_cell_formulas.size(); ++sheet_pos)
+    {
+        spreadsheet::iface::import_sheet* sheet = mp_factory->get_sheet(sheet_pos);
+        if (!sheet)
+            continue;
+
+        spreadsheet::iface::import_formula* xformula = sheet->get_formula();
+        if (!xformula)
+            continue;
+
+        const std::deque<cell_formula_type>& store = m_cell_formulas[sheet_pos];
+        for (const cell_formula_type& cf : store)
+        {
+            xformula->set_position(cf.pos.row, cf.pos.column);
+            xformula->set_formula(ss::formula_grammar_t::xls_xml, cf.formula.data(), cf.formula.size());
+
+            switch (cf.result.type)
+            {
+                case formula_result::result_type::numeric:
+                    xformula->set_result_value(cf.result.value_numeric);
+                    break;
+                case formula_result::result_type::string:
+                case formula_result::result_type::boolean:
+                case formula_result::result_type::empty:
+                    ;
+            }
+
+            xformula->commit();
+        }
     }
 }
 
@@ -1731,6 +1903,13 @@ void xls_xml_context::commit_styles()
         styles->set_xf_horizontal_alignment(style->text_alignment.hor);
         styles->set_xf_vertical_alignment(style->text_alignment.ver);
 
+        if (!style->number_format.empty())
+        {
+            styles->set_number_format_code(style->number_format.data(), style->number_format.size());
+            size_t number_format_id = styles->commit_number_format();
+            styles->set_xf_number_format(number_format_id);
+        }
+
         // TODO : handle text indent level.
 
         size_t xf_id = styles->commit_cell_xf();
@@ -1806,6 +1985,18 @@ xls_xml_context::array_formulas_type& xls_xml_context::get_array_formula_store()
     return m_array_formulas;
 }
 
+void xls_xml_context::store_cell_formula(const pstring& formula, const formula_result& res)
+{
+    assert(m_cur_sheet < ss::sheet_t(m_cell_formulas.size()));
+
+    cell_formula_type cf;
+    cf.pos = get_current_pos();
+    cf.formula = formula;
+    cf.result = res;
+    std::deque<cell_formula_type>& store = m_cell_formulas[m_cur_sheet];
+    store.push_back(std::move(cf));
+}
+
 }
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
