@@ -13,8 +13,12 @@
 #include "orcus/spreadsheet/view_types.hpp"
 #include "orcus/string_pool.hpp"
 
+#include "formula_result.hpp"
+
 #include <string>
 #include <unordered_map>
+#include <list>
+#include <deque>
 
 namespace orcus {
 
@@ -24,9 +28,12 @@ class import_factory;
 class import_sheet;
 class import_sheet_properties;
 class import_named_expression;
+class import_array_formula;
 
 }}
 
+class xls_xml_context;
+
 /**
  * Context for handling <Data> element scopes.
  */
@@ -54,12 +61,7 @@ class xls_xml_data_context : public xml_context_base
 
     enum cell_type { ct_unknown = 0, ct_string, ct_number, ct_datetime };
 
-    spreadsheet::iface::import_factory* mp_factory;
-    spreadsheet::iface::import_sheet* mp_cur_sheet;
-    spreadsheet::row_t m_row;
-    spreadsheet::col_t m_col;
-
-    pstring m_cell_formula;
+    xls_xml_context& m_parent_cxt;
 
     cell_type m_cell_type;
     std::vector<string_segment_type> m_cell_string;
@@ -71,7 +73,7 @@ class xls_xml_data_context : public xml_context_base
     date_time_t m_cell_datetime;
 
 public:
-    xls_xml_data_context(session_context& session_cxt, const tokens& tokens, spreadsheet::iface::import_factory* factory);
+    xls_xml_data_context(session_context& session_cxt, const tokens& tokens, xls_xml_context& parent_cxt);
     virtual ~xls_xml_data_context() override;
 
     virtual bool can_handle_element(xmlns_id_t ns, xml_token_t name) const override;
@@ -86,23 +88,41 @@ public:
      * Intendted to be called from the parent context instance, to reset its
      * internal state before its use.
      */
-    void reset(
-        spreadsheet::iface::import_sheet* sheet,
-        spreadsheet::row_t row, spreadsheet::col_t col,
-        const pstring& cell_formula);
+    void reset();
 
 private:
 
     void start_element_data(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
     void end_element_data();
 
-    void push_formula_cell();
+    bool handle_array_formula_result();
+    void push_array_result(
+        range_formula_results& res, size_t row_offset, size_t col_offset);
 
+    void push_formula_cell(const pstring& formula);
+    void store_array_formula_parent_cell(const pstring& formula);
     void update_current_format();
 };
 
 class xls_xml_context : public xml_context_base
 {
+    friend class xls_xml_data_context;
+
+    struct cell_formula_type
+    {
+        spreadsheet::address_t pos;
+        pstring formula;
+        formula_result result;
+    };
+
+    struct array_formula_type
+    {
+        pstring formula;
+        range_formula_results results;
+
+        array_formula_type(const spreadsheet::range_t& _range, const pstring& _formula);
+    };
+
     struct border_style_type
     {
         spreadsheet::border_direction_t dir = spreadsheet::border_direction_t::unknown;
@@ -143,6 +163,7 @@ class xls_xml_context : public xml_context_base
         font_style_type font;
         fill_style_type fill;
         text_alignment_type text_alignment;
+        pstring number_format;
         std::vector<border_style_type> borders;
     };
 
@@ -183,35 +204,53 @@ class xls_xml_context : public xml_context_base
         spreadsheet::address_t get_top_left_cell() const;
     };
 
+    struct table_properties
+    {
+        spreadsheet::address_t pos; // top-left position
+
+        table_properties();
+        void reset();
+    };
+
     using named_expressions_type = std::vector<named_exp>;
     using styles_type = std::vector<std::unique_ptr<style_type>>;
     using style_id_xf_map_type = std::unordered_map<pstring, size_t, pstring::hash>;
+    using array_formula_pair_type = std::pair<spreadsheet::range_t, std::unique_ptr<array_formula_type>>;
+    using array_formulas_type = std::list<array_formula_pair_type>;
+    using cell_formulas_type = std::deque<std::deque<cell_formula_type>>;
 
 public:
     xls_xml_context(session_context& session_cxt, const tokens& tokens, spreadsheet::iface::import_factory* factory);
     virtual ~xls_xml_context();
 
-    virtual bool can_handle_element(xmlns_id_t ns, xml_token_t name) const;
-    virtual xml_context_base* create_child_context(xmlns_id_t ns, xml_token_t name);
-    virtual void end_child_context(xmlns_id_t ns, xml_token_t name, xml_context_base* child);
+    virtual void declaration(const xml_declaration_t& decl) override;
 
-    virtual void start_element(xmlns_id_t ns, xml_token_t name, const xml_attrs_t& attrs);
-    virtual bool end_element(xmlns_id_t ns, xml_token_t name);
-    virtual void characters(const pstring& str, bool transient);
+    virtual bool can_handle_element(xmlns_id_t ns, xml_token_t name) const override;
+    virtual xml_context_base* create_child_context(xmlns_id_t ns, xml_token_t name) override;
+    virtual void end_child_context(xmlns_id_t ns, xml_token_t name, xml_context_base* child) override;
+
+    virtual void start_element(xmlns_id_t ns, xml_token_t name, const xml_attrs_t& attrs) override;
+    virtual bool end_element(xmlns_id_t ns, xml_token_t name) override;
+    virtual void characters(const pstring& str, bool transient) override;
 
 private:
     void start_element_borders(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
     void start_element_border(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
+    void start_element_number_format(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
     void start_element_cell(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
     void start_element_column(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
     void start_element_row(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
+    void start_element_table(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
+    void start_element_worksheet(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
 
     void end_element_borders();
     void end_element_border();
+    void end_element_number_format();
     void end_element_cell();
     void end_element_column();
     void end_element_row();
-
+    void end_element_table();
+    void end_element_worksheet();
     void end_element_workbook();
     void end_element_styles();
     void end_element_pane();
@@ -221,6 +260,19 @@ private:
     void commit_default_style();
     void commit_styles();
 
+    void push_all_array_formulas();
+
+private:
+    spreadsheet::iface::import_factory* get_import_factory();
+    spreadsheet::iface::import_sheet* get_import_sheet();
+    spreadsheet::address_t get_current_pos() const;
+    pstring pop_and_clear_formula();
+    bool is_array_formula() const;
+    const spreadsheet::range_t& get_array_range() const;
+    array_formulas_type& get_array_formula_store();
+
+    void store_cell_formula(const pstring& formula, const formula_result& res);
+
 private:
     spreadsheet::iface::import_factory* mp_factory;
     spreadsheet::iface::import_sheet* mp_cur_sheet;
@@ -234,18 +286,21 @@ private:
     spreadsheet::col_t m_cur_prop_col; /// current column position for column properties.
     spreadsheet::row_t m_cur_merge_down;
     spreadsheet::col_t m_cur_merge_across;
+    spreadsheet::range_t m_cur_array_range;
     pstring m_cur_cell_formula;
     pstring m_cur_cell_style_id;
 
+    cell_formulas_type m_cell_formulas;
+    array_formulas_type m_array_formulas;
     named_expressions_type m_named_exps_global;
     named_expressions_type m_named_exps_sheet;
     selection m_cursor_selection; /// cursor selection in a single pane.
     split_pane m_split_pane;
-    spreadsheet::sheet_pane_t m_active_pane;
 
     std::unique_ptr<style_type> m_current_style;
     std::unique_ptr<style_type> m_default_style;
     styles_type m_styles;
+    table_properties m_table_props;
 
     style_id_xf_map_type m_style_map;
 
