@@ -115,8 +115,8 @@ void xlsx_autofilter_context::characters(const pstring& /*str*/, bool /*transien
 
 void xlsx_autofilter_context::push_to_model(spreadsheet::iface::import_auto_filter& af) const
 {
-    spreadsheet::range_t range = m_resolver.resolve_range(m_ref_range.data(), m_ref_range.size());
-    af.set_range(range);
+    spreadsheet::src_range_t range = m_resolver.resolve_range(m_ref_range.data(), m_ref_range.size());
+    af.set_range(to_rc_range(range));
 
     column_filters_type::const_iterator it = m_column_filters.begin(), it_end = m_column_filters.end();
     for (; it != it_end; ++it)
