@@ -10,6 +10,7 @@
 
 #include "orcus/spreadsheet/types.hpp"
 #include "orcus/string_pool.hpp"
+#include "orcus/cell_buffer.hpp"
 
 #include "xml_context_base.hpp"
 #include "xlsx_types.hpp"
@@ -42,6 +43,7 @@ public:
 private:
     spreadsheet::iface::import_shared_strings* mp_strings;
     string_pool m_pool;
+    cell_buffer m_cell_buffer;
     pstring m_cur_str;
     bool m_in_segments;
 };
@@ -65,11 +67,15 @@ public:
     virtual void characters(const pstring& str, bool transient);
 
 private:
+    void start_element_number_format(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
+
     void start_element_border(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
     void start_element_diagonal(const xml_token_pair_t& parent, const xml_attrs_t& attrs);
     void start_border_color(const xml_attrs_t& attrs);
     void start_font_color(const xml_attrs_t& attrs);
 
+    void end_element_number_format();
+
 private:
     spreadsheet::iface::import_styles* mp_styles;
     string_pool m_pool;
