@@ -9,6 +9,7 @@
 #define INCLUDED_ORCUS_XLSX_SESSION_DATA_HPP
 
 #include "session_context.hpp"
+#include "formula_result.hpp"
 
 #include "orcus/spreadsheet/types.hpp"
 
@@ -28,19 +29,25 @@ struct xlsx_session_data : public session_context::custom_data
     struct formula
     {
         spreadsheet::sheet_t sheet;
-        spreadsheet::row_t row;
-        spreadsheet::col_t column;
+        spreadsheet::address_t ref;
         std::string exp;
-        std::string range;
-        bool array;
 
         formula(
             spreadsheet::sheet_t _sheet, spreadsheet::row_t _row, spreadsheet::col_t _column,
             const std::string& _exp);
+    };
 
-        formula(
-            spreadsheet::sheet_t _sheet, spreadsheet::row_t _row, spreadsheet::col_t _column,
-            const std::string& _exp, const std::string& _range);
+    struct array_formula
+    {
+        spreadsheet::sheet_t sheet;
+        spreadsheet::range_t ref;
+        std::string exp;
+
+        std::shared_ptr<range_formula_results> results;
+
+        array_formula(
+            spreadsheet::sheet_t sheet, const spreadsheet::range_t& ref,
+            const std::string& exp);
     };
 
     struct shared_formula
@@ -50,7 +57,6 @@ struct xlsx_session_data : public session_context::custom_data
         spreadsheet::col_t column;
         size_t identifier;
         std::string formula;
-        std::string range;
         bool master;
 
         shared_formula(
@@ -59,25 +65,19 @@ struct xlsx_session_data : public session_context::custom_data
 
         shared_formula(
             spreadsheet::sheet_t _sheet, spreadsheet::row_t _row, spreadsheet::col_t _column,
-            size_t _identifier, const std::string& _formula, const std::string& _range);
+            size_t _identifier, const std::string& _formula);
     };
 
     typedef std::vector<std::unique_ptr<formula>> formulas_type;
+    typedef std::vector<std::unique_ptr<array_formula>> array_formulas_type;
     typedef std::vector<std::unique_ptr<shared_formula>> shared_formulas_type;
     typedef std::unordered_map<pstring, spreadsheet::sheet_t, pstring::hash> sheet_name_map_type;
 
     formulas_type m_formulas;
+    array_formulas_type m_array_formulas;
     shared_formulas_type m_shared_formulas;
-    sheet_name_map_type m_sheet_name_map;
 
     virtual ~xlsx_session_data();
-
-    /**
-     * @param name sheet name. Note that this must be already interned with
-     *             the string pool of the session context.
-     * @param id sheet index.
-     */
-    void set_sheet_name_map(pstring name, spreadsheet::sheet_t id);
 };
 
 }
