@@ -17,6 +17,7 @@ namespace orcus {
 namespace spreadsheet { namespace iface {
 
 class import_table;
+class import_reference_resolver;
 
 }}
 
@@ -24,7 +25,9 @@ class xlsx_table_context : public xml_context_base
 {
 public:
     xlsx_table_context(
-        session_context& session_cxt, const tokens& tokens, spreadsheet::iface::import_table& table);
+        session_context& session_cxt, const tokens& tokens,
+        spreadsheet::iface::import_table& table,
+        spreadsheet::iface::import_reference_resolver& resolver);
     virtual ~xlsx_table_context();
 
     virtual bool can_handle_element(xmlns_id_t ns, xml_token_t name) const;
@@ -37,6 +40,7 @@ public:
 
 private:
     spreadsheet::iface::import_table& m_table;
+    spreadsheet::iface::import_reference_resolver& m_resolver;
     std::unique_ptr<xml_context_base> mp_child;
 };
 
