@@ -6,6 +6,7 @@
  */
 
 #include "xml_map_tree.hpp"
+#include "xpath_parser.hpp"
 #include "orcus/global.hpp"
 
 #define ORCUS_DEBUG_XML_MAP_TREE 0
@@ -17,6 +18,7 @@
 #include <cassert>
 #include <algorithm>
 #include <iostream>
+#include <sstream>
 
 using namespace std;
 
@@ -24,102 +26,6 @@ namespace orcus {
 
 namespace {
 
-class xpath_parser
-{
-    const xmlns_context& m_cxt;
-    const char* mp_char;
-    const char* mp_end;
-
-    enum class token_type { element, attribute };
-    token_type m_next_token_type;
-
-public:
-
-    struct token
-    {
-        xmlns_id_t ns;
-        pstring name;
-        bool attribute;
-
-        token(xmlns_id_t _ns, const pstring& _name, bool _attribute) :
-            ns(_ns), name(_name), attribute(_attribute)
-        {
-#if ORCUS_DEBUG_XML_MAP_TREE
-            cout << "xpath_parser::token: (ns='" << (ns ? ns : "none") << "', name='" << name << "', attribute=" << attribute << ")" << endl;
-#endif
-        }
-
-        token() : ns(XMLNS_UNKNOWN_ID), attribute(false) {}
-        token(const token& r) : ns(r.ns), name(r.name), attribute(r.attribute) {}
-    };
-
-    xpath_parser(const xmlns_context& cxt, const char* p, size_t n) :
-        m_cxt(cxt), mp_char(p), mp_end(p+n), m_next_token_type(token_type::element)
-    {
-        if (!n)
-            throw xml_map_tree::xpath_error("empty path");
-
-        if (*p != '/')
-            throw xml_map_tree::xpath_error("first character must be '/'.");
-
-        ++mp_char;
-    }
-
-    token next()
-    {
-        if (mp_char == mp_end)
-            return token();
-
-        const char* p0 = nullptr;
-        size_t len = 0;
-        xmlns_id_t ns = XMLNS_UNKNOWN_ID;
-
-        for (; mp_char != mp_end; ++mp_char, ++len)
-        {
-            if (!p0)
-            {
-                p0 = mp_char;
-                len = 0;
-            }
-
-            switch (*mp_char)
-            {
-                case '/':
-                {
-                    // '/' encountered.  Next token is an element name.
-                    if (m_next_token_type == token_type::attribute)
-                        throw xml_map_tree::xpath_error("attribute name should not contain '/'.");
-
-                    m_next_token_type = token_type::element;
-                    ++mp_char; // skip the '/'.
-                    return token(ns, pstring(p0, len), false);
-                }
-                case '@':
-                {
-                    // '@' encountered.  Next token is an attribute name.
-                    m_next_token_type = token_type::attribute;
-                    ++mp_char; // skip the '@'.
-                    return token(ns, pstring(p0, len), false);
-                }
-                case ':':
-                {
-                    // What comes ':' is a namespace. Reset the name and
-                    // convert the namespace to a proper ID.
-                    pstring ns_name(p0, len);
-                    ns = m_cxt.get(ns_name);
-                    p0 = nullptr; // reset the name.
-                }
-                break;
-                default:
-                    ;
-            }
-        }
-
-        // '/' has never been encountered.  It must be the last name in the path.
-        return token(ns, pstring(p0, len), m_next_token_type == token_type::attribute);
-    }
-};
-
 template<typename T>
 void print_element_stack(ostream& os, const T& elem_stack)
 {
@@ -127,13 +33,14 @@ void print_element_stack(ostream& os, const T& elem_stack)
     for (; it != it_end; ++it)
     {
         const xml_map_tree::element& elem = **it;
-        os << '/' << elem.name;
+        os << '/' << elem.name.name;
     }
 }
 
 }
 
-xml_map_tree::xpath_error::xpath_error(const string& msg) : general_error(msg) {}
+xml_map_tree::range_field_link::range_field_link(const pstring& _xpath, const pstring& _label) :
+    xpath(_xpath), label(_label) {}
 
 xml_map_tree::element_position::element_position() :
     open_begin(0), open_end(0), close_begin(0), close_end(0) {}
@@ -144,20 +51,20 @@ xml_map_tree::range_reference::range_reference(const cell_position& _pos) :
     pos(_pos), row_position(0) {}
 
 xml_map_tree::linkable::linkable(
-    xml_map_tree& parent, xmlns_id_t _ns, const pstring& _name, linkable_node_type _node_type, reference_type _ref_type) :
-    ns(_ns), name(_name), node_type(_node_type), ref_type(_ref_type)
+    xml_map_tree& parent, const xml_name_t& _name, linkable_node_type _node_type, reference_type _ref_type) :
+    name(_name), node_type(_node_type), ref_type(_ref_type)
 {
     parent.create_ref_store(*this);
 }
 
 xml_map_tree::attribute::attribute(args_type args) :
-    linkable(std::get<0>(args), std::get<1>(args), std::get<2>(args), node_attribute, std::get<3>(args)) {}
+    linkable(std::get<0>(args), xml_name_t(std::get<1>(args)), node_attribute, std::get<2>(args)) {}
 
 xml_map_tree::attribute::~attribute() {}
 
 xml_map_tree::element::element(args_type args) :
-    linkable(std::get<0>(args), std::get<1>(args), std::get<2>(args), node_element, std::get<4>(args)),
-    elem_type(std::get<3>(args)),
+    linkable(std::get<0>(args), std::get<1>(args), node_element, std::get<3>(args)),
+    elem_type(std::get<2>(args)),
     child_elements(nullptr),
     range_parent(nullptr),
     row_group(nullptr),
@@ -176,7 +83,7 @@ xml_map_tree::element::element(args_type args) :
 
 xml_map_tree::element::~element() {}
 
-xml_map_tree::element* xml_map_tree::element::get_child(xmlns_id_t _ns, const pstring& _name)
+xml_map_tree::element* xml_map_tree::element::get_child(const xml_name_t& _name)
 {
     if (elem_type != element_unlinked)
         return nullptr;
@@ -185,9 +92,9 @@ xml_map_tree::element* xml_map_tree::element::get_child(xmlns_id_t _ns, const ps
 
     auto it = std::find_if(
         child_elements->begin(), child_elements->end(),
-        [&_ns,&_name](const element* p) -> bool
+        [&_name](const element* p) -> bool
         {
-            return p->ns == _ns && p->name == _name;
+            return p->name == _name;
         }
     );
 
@@ -195,13 +102,13 @@ xml_map_tree::element* xml_map_tree::element::get_child(xmlns_id_t _ns, const ps
 }
 
 xml_map_tree::element* xml_map_tree::element::get_or_create_child(
-    xml_map_tree& parent, xmlns_id_t _ns, const pstring& _name)
+    xml_map_tree& parent, const xml_name_t& _name)
 {
     auto it = std::find_if(
         child_elements->begin(), child_elements->end(),
-        [&_ns,&_name](const element* p) -> bool
+        [&_name](const element* p) -> bool
         {
-            return p->ns == _ns && p->name == _name;
+            return p->name == _name;
         }
     );
 
@@ -211,13 +118,12 @@ xml_map_tree::element* xml_map_tree::element::get_or_create_child(
     string_pool& sp = parent.m_names;
 
     // Insert a new element of this name.
-    auto const nm = sp.intern(_name.get(), _name.size()).first; // work around LLVM < 7 libc++ bug
+    auto const nm = sp.intern(_name.name.get(), _name.name.size()).first; // work around LLVM < 7 libc++ bug
     child_elements->push_back(
         parent.m_element_pool.construct(
             element::args_type(
                 parent,
-                _ns,
-                nm,
+                xml_name_t(_name.ns, nm),
                 element_unlinked,
                 reference_unknown
             )
@@ -228,13 +134,26 @@ xml_map_tree::element* xml_map_tree::element::get_or_create_child(
 }
 
 xml_map_tree::element* xml_map_tree::element::get_or_create_linked_child(
-    xml_map_tree& parent, xmlns_id_t _ns, const pstring& _name, reference_type _ref_type)
+    xml_map_tree& parent, const xml_name_t& _name, reference_type _ref_type)
 {
+    if (!child_elements)
+    {
+        assert(elem_type == element_linked);
+        std::ostringstream os;
+        constexpr xml_name_t::to_string_type type = xml_name_t::use_alias;
+
+        os << "You can't add a child element under an already linked element (this='"
+            << name.to_string(parent.m_xmlns_cxt, type) << "'; child='"
+            << _name.to_string(parent.m_xmlns_cxt, type) << "')";
+
+        throw invalid_map_error(os.str());
+    }
+
     auto it = std::find_if(
         child_elements->begin(), child_elements->end(),
-        [&_ns,&_name](const element* p) -> bool
+        [&_name](const element* p) -> bool
         {
-            return p->ns == _ns && p->name == _name;
+            return p->name == _name;
         }
     );
 
@@ -252,13 +171,12 @@ xml_map_tree::element* xml_map_tree::element::get_or_create_linked_child(
     string_pool& sp = parent.m_names;
 
     // Insert a new linked element of this name.
-    auto const nm = sp.intern(_name.get(), _name.size()).first; // work around LLVM < 7 libc++ bug
+    auto const nm = sp.intern(_name.name.get(), _name.name.size()).first; // work around LLVM < 7 libc++ bug
     child_elements->push_back(
         parent.m_element_pool.construct(
             element::args_type(
                 parent,
-                _ns,
-                nm,
+                xml_name_t(_name.ns, nm),
                 element_linked,
                 _ref_type
             )
@@ -294,12 +212,12 @@ void xml_map_tree::walker::reset()
     m_unlinked_stack.clear();
 }
 
-xml_map_tree::element* xml_map_tree::walker::push_element(xmlns_id_t ns, const pstring& name)
+xml_map_tree::element* xml_map_tree::walker::push_element(const xml_name_t& name)
 {
     if (!m_unlinked_stack.empty())
     {
         // We're still in the unlinked region.
-        m_unlinked_stack.push_back(xml_name_t(ns, name));
+        m_unlinked_stack.push_back(name);
         return nullptr;
     }
 
@@ -308,15 +226,15 @@ xml_map_tree::element* xml_map_tree::walker::push_element(xmlns_id_t ns, const p
         if (!m_parent.mp_root)
         {
             // Tree is empty.
-            m_unlinked_stack.push_back(xml_name_t(ns, name));
+            m_unlinked_stack.push_back(name);
             return nullptr;
         }
 
         element* p = m_parent.mp_root;
-        if (p->ns != ns || p->name != name)
+        if (p->name != name)
         {
             // Names differ.
-            m_unlinked_stack.push_back(xml_name_t(ns, name));
+            m_unlinked_stack.push_back(name);
             return nullptr;
         }
 
@@ -327,7 +245,7 @@ xml_map_tree::element* xml_map_tree::walker::push_element(xmlns_id_t ns, const p
     if (m_stack.back()->elem_type == element_unlinked)
     {
         // Check if the current element has a child of the same name.
-        element* p = m_stack.back()->get_child(ns, name);
+        element* p = m_stack.back()->get_child(name);
         if (p)
         {
             m_stack.push_back(p);
@@ -335,16 +253,16 @@ xml_map_tree::element* xml_map_tree::walker::push_element(xmlns_id_t ns, const p
         }
     }
 
-    m_unlinked_stack.push_back(xml_name_t(ns, name));
+    m_unlinked_stack.push_back(name);
     return nullptr;
 }
 
-xml_map_tree::element* xml_map_tree::walker::pop_element(xmlns_id_t ns, const pstring& name)
+xml_map_tree::element* xml_map_tree::walker::pop_element(const xml_name_t& name)
 {
     if (!m_unlinked_stack.empty())
     {
         // We're in the unlinked region.  Pop element from the unlinked stack.
-        if (m_unlinked_stack.back().ns != ns || m_unlinked_stack.back().name != name)
+        if (m_unlinked_stack.back() != name)
             throw general_error("Closing element has a different name than the opening element. (unlinked stack)");
 
         m_unlinked_stack.pop_back();
@@ -359,7 +277,7 @@ xml_map_tree::element* xml_map_tree::walker::pop_element(xmlns_id_t ns, const ps
     if (m_stack.empty())
         throw general_error("Element was popped while the stack was empty.");
 
-    if (m_stack.back()->ns != ns || m_stack.back()->name != name)
+    if (m_stack.back()->name != name)
         throw general_error("Closing element has a different name than the opening element. (linked stack)");
 
     m_stack.pop_back();
@@ -367,19 +285,24 @@ xml_map_tree::element* xml_map_tree::walker::pop_element(xmlns_id_t ns, const ps
 }
 
 xml_map_tree::xml_map_tree(xmlns_repository& xmlns_repo) :
-    m_xmlns_cxt(xmlns_repo.create_context()), mp_root(nullptr) {}
+    m_xmlns_cxt(xmlns_repo.create_context()),
+    mp_root(nullptr),
+    m_default_ns(XMLNS_UNKNOWN_ID) {}
 
 xml_map_tree::~xml_map_tree() {}
 
-void xml_map_tree::set_namespace_alias(const pstring& alias, const pstring& uri)
+void xml_map_tree::set_namespace_alias(const pstring& alias, const pstring& uri, bool default_ns)
 {
 #if ORCUS_DEBUG_XML_MAP_TREE
-    cout << "xml_map_tree::set_namespace_alias: alias='" << alias << "', uri='" << uri << "'" << endl;
+    cout << "xml_map_tree::set_namespace_alias: alias='" << alias << "', uri='" << uri << "', default=" << default_ns << endl;
 #endif
     // We need to turn the alias string persistent because the xmlns context
     // doesn't intern the alias strings.
     pstring alias_safe = m_names.intern(alias).first;
-    m_xmlns_cxt.push(alias_safe, uri);
+    xmlns_id_t ns = m_xmlns_cxt.push(alias_safe, uri);
+
+    if (default_ns)
+        m_default_ns = ns;
 }
 
 xmlns_id_t xml_map_tree::get_namespace(const pstring& alias) const
@@ -423,18 +346,18 @@ void xml_map_tree::start_range(const cell_position& pos)
     m_cur_range_pos = pos;
 }
 
-void xml_map_tree::append_range_field_link(const pstring& xpath)
+void xml_map_tree::append_range_field_link(const pstring& xpath, const pstring& label)
 {
     if (xpath.empty())
         return;
 
-    m_cur_range_field_links.emplace_back(xpath);
+    m_cur_range_field_links.emplace_back(xpath, label);
 }
 
 void xml_map_tree::insert_range_field_link(
-    range_reference& range_ref, element_list_type& range_parent, const pstring& xpath)
+    range_reference& range_ref, element_list_type& range_parent, const range_field_link& field)
 {
-    linked_node_type linked_node = get_linked_node(xpath, reference_range_field);
+    linked_node_type linked_node = get_linked_node(field.xpath, reference_range_field);
     if (linked_node.elem_stack.size() < 2)
         throw xpath_error("Path of a range field link must be at least 2 levels.");
 
@@ -444,6 +367,9 @@ void xml_map_tree::insert_range_field_link(
     if (linked_node.anchor_elem)
         linked_node.anchor_elem->linked_range_fields.push_back(range_ref.field_nodes.size());
 
+    if (!field.label.empty())
+        linked_node.node->label = intern_string(field.label);
+
     switch (linked_node.node->node_type)
     {
         case node_element:
@@ -454,8 +380,8 @@ void xml_map_tree::insert_range_field_link(
             p->field_ref->column_pos = range_ref.field_nodes.size();
 
             range_ref.field_nodes.push_back(p);
+            break;
         }
-        break;
         case node_attribute:
         {
             attribute* p = static_cast<attribute*>(linked_node.node);
@@ -464,8 +390,8 @@ void xml_map_tree::insert_range_field_link(
             p->field_ref->column_pos = range_ref.field_nodes.size();
 
             range_ref.field_nodes.push_back(p);
+            break;
         }
-        break;
         default:
             ;
     }
@@ -537,14 +463,13 @@ void xml_map_tree::commit_range()
     // commont parent element for this range.
     element_list_type range_parent;
 
-    for (const pstring& path : m_cur_range_field_links)
-        insert_range_field_link(*range_ref, range_parent, path);
+    for (const range_field_link& field : m_cur_range_field_links)
+        insert_range_field_link(*range_ref, range_parent, field);
 
 #if ORCUS_DEBUG_XML_MAP_TREE
     cout << "parent element path for this range: ";
-    element_list_type::iterator it = range_parent.begin(), it_end = range_parent.end();
-    for (; it != it_end; ++it)
-        cout << "/" << (**it).name;
+    for (const element* elem : range_parent)
+        cout << "/" << elem->name.to_string(m_xmlns_cxt, xml_name_t::use_alias);
     cout << endl;
 #endif
 
@@ -570,11 +495,11 @@ const xml_map_tree::linkable* xml_map_tree::get_link(const pstring& xpath) const
 #endif
     const linkable* cur_node = mp_root;
 
-    xpath_parser parser(m_xmlns_cxt, xpath.get(), xpath.size());
+    xpath_parser parser(m_xmlns_cxt, xpath.get(), xpath.size(), m_default_ns);
 
     // Check the root element first.
     xpath_parser::token token = parser.next();
-    if (cur_node->ns != token.ns || cur_node->name != token.name)
+    if (cur_node->name.ns != token.ns || cur_node->name.name != token.name)
         // Root element name doesn't match.
         return nullptr;
 
@@ -595,7 +520,7 @@ const xml_map_tree::linkable* xml_map_tree::get_link(const pstring& xpath) const
                 attrs.begin(), attrs.end(),
                 [&token](const attribute* p) -> bool
                 {
-                    return p->ns == token.ns && p->name == token.name;
+                    return p->name.ns == token.ns && p->name.name == token.name;
                 }
             );
 
@@ -622,7 +547,7 @@ const xml_map_tree::linkable* xml_map_tree::get_link(const pstring& xpath) const
             elem->child_elements->begin(), elem->child_elements->end(),
             [&token](const element* p) -> bool
             {
-                return p->ns == token.ns && p->name == token.name;
+                return p->name.ns == token.ns && p->name.name == token.name;
             }
         );
 
@@ -695,14 +620,14 @@ xml_map_tree::linked_node_type xml_map_tree::get_linked_node(const pstring& xpat
     linked_node_type ret;
 
     assert(!xpath.empty());
-    xpath_parser parser(m_xmlns_cxt, xpath.get(), xpath.size());
+    xpath_parser parser(m_xmlns_cxt, xpath.get(), xpath.size(), m_default_ns);
 
     // Get the root element first.
     xpath_parser::token token = parser.next();
     if (mp_root)
     {
         // Make sure the root element's names are the same.
-        if (mp_root->ns != token.ns || mp_root->name != token.name)
+        if (mp_root->name.ns != token.ns || mp_root->name.name != token.name)
             throw xpath_error("path begins with inconsistent root level name.");
     }
     else
@@ -715,8 +640,7 @@ xml_map_tree::linked_node_type xml_map_tree::get_linked_node(const pstring& xpat
         mp_root = m_element_pool.construct(
             element::args_type(
                 *this,
-                token.ns,
-                nm,
+                xml_name_t(token.ns, nm),
                 element_unlinked,
                 reference_unknown
             )
@@ -737,7 +661,7 @@ xml_map_tree::linked_node_type xml_map_tree::get_linked_node(const pstring& xpat
         if (token.attribute)
             throw xpath_error("attribute must always be at the end of the path.");
 
-        cur_element = cur_element->get_or_create_child(*this, token.ns, token.name);
+        cur_element = cur_element->get_or_create_child(*this, {token.ns, token.name});
         ret.elem_stack.push_back(cur_element);
         token = token_next;
 
@@ -759,7 +683,7 @@ xml_map_tree::linked_node_type xml_map_tree::get_linked_node(const pstring& xpat
             attrs.begin(), attrs.end(),
             [&token](const attribute* p) -> bool
             {
-                return p->ns == token.ns && p->name == token.name;
+                return p->name.ns == token.ns && p->name.name == token.name;
             }
         );
 
@@ -770,8 +694,7 @@ xml_map_tree::linked_node_type xml_map_tree::get_linked_node(const pstring& xpat
         attribute* p = m_attribute_pool.construct(
             attribute::args_type(
                 *this,
-                token.ns,
-                nm,
+                xml_name_t(token.ns, nm),
                 ref_type
             )
         );
@@ -781,7 +704,7 @@ xml_map_tree::linked_node_type xml_map_tree::get_linked_node(const pstring& xpat
     }
     else
     {
-        element* elem = cur_element->get_or_create_linked_child(*this, token.ns, token.name, ref_type);
+        element* elem = cur_element->get_or_create_linked_child(*this, {token.ns, token.name}, ref_type);
         ret.elem_stack.push_back(elem);
         ret.node = elem;
 
@@ -796,14 +719,14 @@ xml_map_tree::linked_node_type xml_map_tree::get_linked_node(const pstring& xpat
 xml_map_tree::element* xml_map_tree::get_element(const pstring& xpath)
 {
     assert(!xpath.empty());
-    xpath_parser parser(m_xmlns_cxt, xpath.get(), xpath.size());
+    xpath_parser parser(m_xmlns_cxt, xpath.get(), xpath.size(), m_default_ns);
 
     // Get the root element first.
     xpath_parser::token token = parser.next();
     if (mp_root)
     {
         // Make sure the root element's names are the same.
-        if (mp_root->ns != token.ns || mp_root->name != token.name)
+        if (mp_root->name.ns != token.ns || mp_root->name.name != token.name)
             throw xpath_error("path begins with inconsistent root level name.");
     }
     else
@@ -816,8 +739,7 @@ xml_map_tree::element* xml_map_tree::get_element(const pstring& xpath)
         mp_root = m_element_pool.construct(
             element::args_type(
                 *this,
-                token.ns,
-                nm,
+                xml_name_t(token.ns, nm),
                 element_unlinked,
                 reference_unknown
             )
@@ -833,7 +755,7 @@ xml_map_tree::element* xml_map_tree::get_element(const pstring& xpath)
         if (token.attribute)
             throw xpath_error("attribute was not expected.");
 
-        cur_element = cur_element->get_or_create_child(*this, token.ns, token.name);
+        cur_element = cur_element->get_or_create_child(*this, {token.ns, token.name});
     }
 
     assert(cur_element);
@@ -844,7 +766,7 @@ std::ostream& operator<< (std::ostream& os, const xml_map_tree::linkable& link)
 {
     if (!link.ns_alias.empty())
         os << link.ns_alias << ':';
-    os << link.name;
+    os << link.name.name;
     return os;
 }
 
