@@ -54,6 +54,13 @@ const char* help_dump_check =
 const char* help_debug =
 "Turn on a debug mode and optionally specify a debug level in order to generate run-time debug outputs.";
 
+const char* help_recalc =
+"Re-calculate all formula cells after the documetn is loaded.";
+
+const char* help_formula_error_policy =
+"Specify whether to abort immediately when the loader fails to parse the first "
+"formula cell ('fail'), or skip the offending cells and continue ('skip').";
+
 const char* help_row_size =
 "Specify the number of maximum rows in each sheet.";
 
@@ -107,10 +114,15 @@ bool parse_import_filter_args(
     iface::import_filter& app, iface::document_dumper& doc,
     extra_args_handler* args_handler)
 {
+    bool debug = false;
+    bool recalc_formula_cells = false;
+
     po::options_description desc("Allowed options");
     desc.add_options()
         ("help,h", "Print this help.")
-        ("debug,d", po::value<uint16_t>()->default_value(0u)->implicit_value(1u), help_debug)
+        ("debug,d", po::bool_switch(&debug), help_debug)
+        ("recalc,r", po::bool_switch(&recalc_formula_cells), help_recalc)
+        ("error-policy,e", po::value<string>()->default_value("fail"), help_formula_error_policy)
         ("dump-check", help_dump_check)
         ("output,o", po::value<string>(), help_output)
         ("output-format,f", po::value<string>(), gen_help_output_format().data())
@@ -169,6 +181,18 @@ bool parse_import_filter_args(
     if (vm.count("row-size"))
         fact.set_default_row_size(vm["row-size"].as<spreadsheet::row_t>());
 
+    std::string error_policy_s = vm["error-policy"].as<std::string>();
+    spreadsheet::formula_error_policy_t error_policy =
+        spreadsheet::to_formula_error_policy(error_policy_s.data(), error_policy_s.size());
+
+    if (error_policy == spreadsheet::formula_error_policy_t::unknown)
+    {
+        cerr << "Unrecognized error policy: " << error_policy_s << endl;
+        return false;
+    }
+
+    fact.set_formula_error_policy(error_policy);
+
     if (infile.empty())
     {
         cerr << err_no_input_file << endl;
@@ -176,13 +200,15 @@ bool parse_import_filter_args(
     }
 
     config opt = app.get_config();
-    opt.debug = vm["debug"].as<uint16_t>();
+    opt.debug = debug;
 
     if (args_handler)
         args_handler->map_to_config(opt, vm);
 
     app.set_config(opt);
 
+    fact.set_recalc_formula_cells(recalc_formula_cells);
+
     if (vm.count("dump-check"))
     {
         // 'outdir' is used as the output file path in this mode.
