@@ -43,7 +43,8 @@ void test_csv_import()
 
         std::cout << "checking " << path << "..." << std::endl;
 
-        spreadsheet::document doc;
+        spreadsheet::range_size_t ss{1048576, 16384};
+        spreadsheet::document doc{ss};
         {
             spreadsheet::import_factory factory(doc);
             orcus_csv app(&factory);
@@ -97,17 +98,18 @@ void test_csv_import_split_sheet()
     conf.csv.header_row_size = 0;
     conf.csv.split_to_multiple_sheets = true;
 
-    spreadsheet::document doc;
+    // Set the row size to 11 to make sure the split occurs.
+    spreadsheet::range_size_t ss{11, 4};
+    spreadsheet::document doc{ss};
     {
-        // Set the row size to 11 to make sure the split occurs.
-        spreadsheet::import_factory factory(doc, 11, 4);
+        spreadsheet::import_factory factory(doc);
         orcus_csv app(&factory);
         app.set_config(conf);
 
         app.read_file(path.c_str());
     }
 
-    assert(doc.sheet_size() == 2);
+    assert(doc.get_sheet_count() == 2);
 
     // Dump the content of the model.
     std::string check = test::get_content_check(doc);
@@ -126,15 +128,14 @@ void test_csv_import_split_sheet()
     doc.clear();
     conf.csv.header_row_size = 1;
     {
-        // Set the row size to 11 to make sure the split occurs.
-        spreadsheet::import_factory factory(doc, 11, 4);
+        spreadsheet::import_factory factory(doc);
         orcus_csv app(&factory);
         app.set_config(conf);
 
         app.read_file(path.c_str());
     }
 
-    assert(doc.sheet_size() == 2);
+    assert(doc.get_sheet_count() == 2);
 
     // Dump the content of the model.
     check = test::get_content_check(doc);
@@ -155,14 +156,14 @@ void test_csv_import_split_sheet()
     doc.clear();
 
     {
-        spreadsheet::import_factory factory(doc, 11, 4);
+        spreadsheet::import_factory factory(doc);
         orcus_csv app(&factory);
         app.set_config(conf);
 
         app.read_file(path.c_str());
     }
 
-    assert(doc.sheet_size() == 1);
+    assert(doc.get_sheet_count() == 1);
 
     // Dump the content of the model.
     check = test::get_content_check(doc);
