@@ -42,11 +42,13 @@ config test_config(format_t::xlsx);
 
 std::unique_ptr<spreadsheet::document> load_doc(const pstring& path)
 {
-    std::unique_ptr<spreadsheet::document> doc = orcus::make_unique<spreadsheet::document>();
+    spreadsheet::range_size_t ss{1048576, 16384};
+    std::unique_ptr<spreadsheet::document> doc = orcus::make_unique<spreadsheet::document>(ss);
     spreadsheet::import_factory factory(*doc);
     orcus_xlsx app(&factory);
     app.read_file(path.str());
     app.set_config(test_config);
+    doc->recalc_formula_cells();
 
     return doc;
 }
@@ -101,14 +103,11 @@ void test_xlsx_import()
 
         // Read the input.xlsx document.
         path.append("input.xlsx");
-        spreadsheet::document doc;
-        spreadsheet::import_factory factory(doc);
-        orcus_xlsx app(&factory);
-        app.read_file(path.c_str());
+        auto doc = load_doc(path);
 
         // Dump the content of the model.
         ostringstream os;
-        doc.dump_check(os);
+        doc->dump_check(os);
         string check = os.str();
 
         // Check that against known control.
@@ -128,7 +127,8 @@ void test_xlsx_import()
 void test_xlsx_table_autofilter()
 {
     string path(SRCDIR"/test/xlsx/table/autofilter.xlsx");
-    document doc;
+    spreadsheet::range_size_t ss{1048576, 16384};
+    document doc{ss};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.read_file(path.c_str());
@@ -162,7 +162,7 @@ void test_xlsx_table_autofilter()
 void test_xlsx_table()
 {
     string path(SRCDIR"/test/xlsx/table/table-1.xlsx");
-    document doc;
+    document doc{{1048576, 16384}};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.read_file(path.c_str());
@@ -229,7 +229,8 @@ void test_xlsx_merged_cells()
 {
     string path(SRCDIR"/test/xlsx/merged-cells/simple.xlsx");
 
-    document doc;
+    spreadsheet::range_size_t ss{1048576, 16384};
+    document doc{ss};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.set_config(test_config);
@@ -274,7 +275,7 @@ void test_xlsx_date_time()
 {
     string path(SRCDIR"/test/xlsx/date-time/input.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.set_config(test_config);
@@ -737,7 +738,7 @@ void test_xlsx_hidden_rows_columns()
     // The rest of the rows are visible.
     assert(!sh->is_row_hidden(9, &row_start, &row_end));
     assert(row_start == 9);
-    assert(row_end == sh->row_size()); // the end position is non-inclusive.
+    assert(row_end == doc->get_sheet_size().rows); // the end position is non-inclusive.
 
     sh = doc->get_sheet("Hidden Columns");
     assert(sh);
@@ -767,14 +768,14 @@ void test_xlsx_hidden_rows_columns()
     // The rest of the columns are all visible.
     assert(!sh->is_col_hidden(11, &col_start, &col_end));
     assert(col_start == 11);
-    assert(col_end == sh->col_size()); // non-inclusive
+    assert(col_end == doc->get_sheet_size().columns); // non-inclusive
 }
 
 void test_xlsx_pivot_two_pivot_caches()
 {
     string path(SRCDIR"/test/xlsx/pivot-table/two-pivot-caches.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.set_config(test_config);
@@ -882,7 +883,7 @@ void test_xlsx_pivot_mixed_type_field()
 {
     string path(SRCDIR"/test/xlsx/pivot-table/mixed-type-field.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.set_config(test_config);
@@ -1008,7 +1009,7 @@ void test_xlsx_pivot_group_field()
 {
     string path(SRCDIR"/test/xlsx/pivot-table/group-field.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.set_config(test_config);
@@ -1101,7 +1102,7 @@ void test_xlsx_pivot_group_by_numbers()
 {
     string path(SRCDIR"/test/xlsx/pivot-table/group-by-numbers.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.set_config(test_config);
@@ -1182,7 +1183,7 @@ void test_xlsx_pivot_group_by_dates()
 {
     string path(SRCDIR"/test/xlsx/pivot-table/group-by-dates.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.set_config(test_config);
@@ -1293,7 +1294,7 @@ void test_xlsx_pivot_error_values()
 {
     string path(SRCDIR"/test/xlsx/pivot-table/error-values.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     import_factory factory(doc);
     orcus_xlsx app(&factory);
     app.set_config(test_config);
@@ -1344,7 +1345,7 @@ void test_xlsx_view_cursor_per_sheet()
 {
     string path(SRCDIR"/test/xlsx/view/cursor-per-sheet.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     spreadsheet::view view(doc);
     spreadsheet::import_factory factory(doc, view);
     orcus_xlsx app(&factory);
@@ -1358,11 +1359,12 @@ void test_xlsx_view_cursor_per_sheet()
     const spreadsheet::sheet_view* sv = view.get_sheet_view(0);
     assert(sv);
 
-    spreadsheet::iface::import_reference_resolver* resolver = factory.get_reference_resolver();
+    spreadsheet::iface::import_reference_resolver* resolver =
+        factory.get_reference_resolver(spreadsheet::formula_ref_context_t::global);
     assert(resolver);
 
     // On Sheet1, the cursor should be set to C4.
-    spreadsheet::range_t expected = resolver->resolve_range(ORCUS_ASCII("C4"));
+    spreadsheet::range_t expected = to_rc_range(resolver->resolve_range(ORCUS_ASCII("C4")));
     spreadsheet::range_t actual = sv->get_selection(spreadsheet::sheet_pane_t::top_left);
     assert(expected == actual);
 
@@ -1370,7 +1372,7 @@ void test_xlsx_view_cursor_per_sheet()
     assert(sv);
 
     // On Sheet2, the cursor should be set to D8.
-    expected = resolver->resolve_range(ORCUS_ASCII("D8"));
+    expected = to_rc_range(resolver->resolve_range(ORCUS_ASCII("D8")));
     actual = sv->get_selection(spreadsheet::sheet_pane_t::top_left);
     assert(expected == actual);
 
@@ -1378,7 +1380,7 @@ void test_xlsx_view_cursor_per_sheet()
     assert(sv);
 
     // On Sheet3, the cursor should be set to D2.
-    expected = resolver->resolve_range(ORCUS_ASCII("D2"));
+    expected = to_rc_range(resolver->resolve_range(ORCUS_ASCII("D2")));
     actual = sv->get_selection(spreadsheet::sheet_pane_t::top_left);
     assert(expected == actual);
 
@@ -1386,7 +1388,7 @@ void test_xlsx_view_cursor_per_sheet()
     assert(sv);
 
     // On Sheet4, the cursor should be set to C5:E8.
-    expected = resolver->resolve_range(ORCUS_ASCII("C5:E8"));
+    expected = to_rc_range(resolver->resolve_range(ORCUS_ASCII("C5:E8")));
     actual = sv->get_selection(spreadsheet::sheet_pane_t::top_left);
     assert(expected == actual);
 }
@@ -1402,7 +1404,7 @@ void test_xlsx_view_cursor_split_pane()
 {
     string path(SRCDIR"/test/xlsx/view/cursor-split-pane.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     spreadsheet::view view(doc);
     spreadsheet::import_factory factory(doc, view);
     orcus_xlsx app(&factory);
@@ -1410,7 +1412,8 @@ void test_xlsx_view_cursor_split_pane()
 
     app.read_file(path.c_str());
 
-    spreadsheet::iface::import_reference_resolver* resolver = factory.get_reference_resolver();
+    spreadsheet::iface::import_reference_resolver* resolver =
+        factory.get_reference_resolver(spreadsheet::formula_ref_context_t::global);
     assert(resolver);
 
     // Sheet4 should be active.
@@ -1425,7 +1428,7 @@ void test_xlsx_view_cursor_split_pane()
     assert(sv->get_split_pane().ver_split == 1800.0);
 
     {
-        spreadsheet::address_t expected = resolver->resolve_address(ORCUS_ASCII("F6"));
+        spreadsheet::address_t expected = to_rc_address(resolver->resolve_address(ORCUS_ASCII("F6")));
         spreadsheet::address_t actual = sv->get_split_pane().top_left_cell;
         assert(expected == actual);
     }
@@ -1441,7 +1444,7 @@ void test_xlsx_view_cursor_split_pane()
     for (const expected_selection& es : expected_selections)
     {
         // cursor in the top-left pane.
-        spreadsheet::range_t expected = resolver->resolve_range(es.sel, es.sel_n);
+        spreadsheet::range_t expected = to_rc_range(resolver->resolve_range(es.sel, es.sel_n));
         spreadsheet::range_t actual = sv->get_selection(es.pane);
         assert(expected == actual);
     }
@@ -1455,7 +1458,7 @@ void test_xlsx_view_cursor_split_pane()
     assert(sv->get_split_pane().ver_split == 2400.0);
 
     {
-        spreadsheet::address_t expected = resolver->resolve_address(ORCUS_ASCII("F8"));
+        spreadsheet::address_t expected = to_rc_address(resolver->resolve_address(ORCUS_ASCII("F8")));
         spreadsheet::address_t actual = sv->get_split_pane().top_left_cell;
         assert(expected == actual);
     }
@@ -1471,7 +1474,7 @@ void test_xlsx_view_cursor_split_pane()
     for (const expected_selection& es : expected_selections)
     {
         // cursor in the top-left pane.
-        spreadsheet::range_t expected = resolver->resolve_range(es.sel, es.sel_n);
+        spreadsheet::range_t expected = to_rc_range(resolver->resolve_range(es.sel, es.sel_n));
         spreadsheet::range_t actual = sv->get_selection(es.pane);
         assert(expected == actual);
     }
@@ -1485,7 +1488,7 @@ void test_xlsx_view_cursor_split_pane()
     assert(sv->get_split_pane().ver_split == 1500.0);
 
     {
-        spreadsheet::address_t expected = resolver->resolve_address(ORCUS_ASCII("A5"));
+        spreadsheet::address_t expected = to_rc_address(resolver->resolve_address(ORCUS_ASCII("A5")));
         spreadsheet::address_t actual = sv->get_split_pane().top_left_cell;
         assert(expected == actual);
     }
@@ -1499,7 +1502,7 @@ void test_xlsx_view_cursor_split_pane()
     for (const expected_selection& es : expected_selections)
     {
         // cursor in the top-left pane.
-        spreadsheet::range_t expected = resolver->resolve_range(es.sel, es.sel_n);
+        spreadsheet::range_t expected = to_rc_range(resolver->resolve_range(es.sel, es.sel_n));
         spreadsheet::range_t actual = sv->get_selection(es.pane);
         assert(expected == actual);
     }
@@ -1513,7 +1516,7 @@ void test_xlsx_view_cursor_split_pane()
     assert(sv->get_split_pane().ver_split == 0.0);
 
     {
-        spreadsheet::address_t expected = resolver->resolve_address(ORCUS_ASCII("E1"));
+        spreadsheet::address_t expected = to_rc_address(resolver->resolve_address(ORCUS_ASCII("E1")));
         spreadsheet::address_t actual = sv->get_split_pane().top_left_cell;
         assert(expected == actual);
     }
@@ -1527,7 +1530,7 @@ void test_xlsx_view_cursor_split_pane()
     for (const expected_selection& es : expected_selections)
     {
         // cursor in the top-left pane.
-        spreadsheet::range_t expected = resolver->resolve_range(es.sel, es.sel_n);
+        spreadsheet::range_t expected = to_rc_range(resolver->resolve_range(es.sel, es.sel_n));
         spreadsheet::range_t actual = sv->get_selection(es.pane);
         assert(expected == actual);
     }
@@ -1537,7 +1540,7 @@ void test_xlsx_view_frozen_pane()
 {
     string path(SRCDIR"/test/xlsx/view/frozen-pane.xlsx");
 
-    document doc;
+    document doc{{1048576, 16384}};
     spreadsheet::view view(doc);
     spreadsheet::import_factory factory(doc, view);
     orcus_xlsx app(&factory);
@@ -1545,7 +1548,8 @@ void test_xlsx_view_frozen_pane()
 
     app.read_file(path.c_str());
 
-    spreadsheet::iface::import_reference_resolver* resolver = factory.get_reference_resolver();
+    spreadsheet::iface::import_reference_resolver* resolver =
+        factory.get_reference_resolver(spreadsheet::formula_ref_context_t::global);
     assert(resolver);
 
     // Sheet3 should be active.
@@ -1557,7 +1561,7 @@ void test_xlsx_view_frozen_pane()
     {
         // Sheet1 is vertically frozen between columns A and B.
         const spreadsheet::frozen_pane_t& fp = sv->get_frozen_pane();
-        assert(fp.top_left_cell == resolver->resolve_address(ORCUS_ASCII("B1")));
+        assert(fp.top_left_cell == to_rc_address(resolver->resolve_address(ORCUS_ASCII("B1"))));
         assert(fp.visible_columns == 1);
         assert(fp.visible_rows == 0);
         assert(sv->get_active_pane() == spreadsheet::sheet_pane_t::top_right);
@@ -1569,7 +1573,7 @@ void test_xlsx_view_frozen_pane()
     {
         // Sheet2 is horizontally frozen between rows 1 and 2.
         const spreadsheet::frozen_pane_t& fp = sv->get_frozen_pane();
-        assert(fp.top_left_cell == resolver->resolve_address(ORCUS_ASCII("A2")));
+        assert(fp.top_left_cell == to_rc_address(resolver->resolve_address(ORCUS_ASCII("A2"))));
         assert(fp.visible_columns == 0);
         assert(fp.visible_rows == 1);
         assert(sv->get_active_pane() == spreadsheet::sheet_pane_t::bottom_left);
@@ -1581,7 +1585,7 @@ void test_xlsx_view_frozen_pane()
     {
         // Sheet3 is frozen both horizontally and vertically.
         const spreadsheet::frozen_pane_t& fp = sv->get_frozen_pane();
-        assert(fp.top_left_cell == resolver->resolve_address(ORCUS_ASCII("E9")));
+        assert(fp.top_left_cell == to_rc_address(resolver->resolve_address(ORCUS_ASCII("E9"))));
         assert(fp.visible_columns == 4);
         assert(fp.visible_rows == 8);
         assert(sv->get_active_pane() == spreadsheet::sheet_pane_t::bottom_right);
@@ -1598,7 +1602,7 @@ void test_xlsx_doc_structure_unordered_sheet_positions()
         "S1", "S2", "S3", "S4", "S5", "S6", "S7", "S8", "S9"
     };
 
-    assert(doc->sheet_size() == expected_sheet_names.size());
+    assert(doc->get_sheet_count() == expected_sheet_names.size());
 
     sheet_t n = expected_sheet_names.size();
     for (sheet_t i = 0; i < n; ++i)
