@@ -7,10 +7,15 @@
 
 #include "orcus/parser_global.hpp"
 #include "orcus/cell_buffer.hpp"
+#include "orcus/global.hpp"
+#include "orcus/exception.hpp"
+
+#include "numeric_parser.hpp"
 
 #include <cassert>
 #include <cmath>
 #include <iostream>
+#include <limits>
 
 namespace orcus {
 
@@ -19,7 +24,7 @@ const size_t parse_quoted_string_state::error_illegal_escape_char = 2;
 
 bool is_blank(char c)
 {
-    return is_in(c, " \t\n\r");
+    return is_in(c, ORCUS_ASCII(" \t\n\r"));
 }
 
 bool is_alpha(char c)
@@ -33,7 +38,7 @@ bool is_alpha(char c)
 
 bool is_name_char(char c)
 {
-    return is_in(c, "-_");
+    return is_in(c, ORCUS_ASCII("-_"));
 }
 
 bool is_numeric(char c)
@@ -41,9 +46,16 @@ bool is_numeric(char c)
     return ('0' <= c && c <= '9');
 }
 
-bool is_in(char c, const char* allowed)
+bool is_in(char c, const char* allowed, size_t n_allowed)
 {
-    for (; *allowed != '\0'; ++allowed)
+#ifdef __ORCUS_DEBUG_UTILS
+    if (allowed && !n_allowed)
+        throw std::invalid_argument("'allowed' pointer is non-null but the value of 'n_allowed' is 0.");
+#endif
+
+    const char* p_end = allowed + n_allowed;
+
+    for (; allowed != p_end; ++allowed)
     {
         if (c == *allowed)
             return true;
@@ -61,131 +73,17 @@ void write_to(std::ostringstream& os, const char* p, size_t n)
         os << *p;
 }
 
-namespace {
-
-/**
- * Parse the exponent part of a numeric string.
- *
- * @return extra divisor to multiply to the original divisor, or 0.0 if the
- *         parsing fails.
- */
-double parse_exponent(const char*& p, const char* p_end)
-{
-    const char* p0 = p; // original position to restore to in case of parsing failure.
-    double exponent = 0.0;
-    bool negative_sign = false;
-    bool valid = false;
-
-    // Check for presence of a sign.
-    if (p != p_end)
-    {
-        switch (*p)
-        {
-            case '+':
-                ++p;
-            break;
-            case '-':
-                negative_sign = true;
-                ++p;
-            break;
-            default:
-                ;
-        }
-    }
-
-    for (; p != p_end; ++p)
-    {
-        if (*p < '0' || '9' < *p)
-        {
-            // Non-digit encountered.
-            break;
-        }
-
-        valid = true;
-        exponent *= 10.0;
-        exponent += *p - '0';
-    }
-
-    if (!valid)
-    {
-        // Restore the original position on failed parsing.
-        p = p0;
-        return 0.0;
-    }
-
-    if (!negative_sign)
-        exponent = -exponent;
-
-    return std::pow(10.0, exponent);
-}
-
-}
-
 double parse_numeric(const char*& p, size_t max_length)
 {
+    using numeric_parser_type = detail::numeric_parser<detail::generic_parser_trait>;
+
     const char* p_end = p + max_length;
-    double ret = 0.0, divisor = 1.0;
-    bool negative_sign = false;
-    bool before_decimal_pt = true;
-    bool has_digit = false;
 
-    // Check for presence of a sign.
-    if (p != p_end)
-    {
-        switch (*p)
-        {
-            case '+':
-                ++p;
-            break;
-            case '-':
-                negative_sign = true;
-                ++p;
-            break;
-            default:
-                ;
-        }
-    }
-
-    for (; p != p_end; ++p)
-    {
-        if (*p == '.')
-        {
-            if (!before_decimal_pt)
-            {
-                // Second '.' encountered. Terminate the parsing.
-                ret /= divisor;
-                return negative_sign ? -ret : ret;
-            }
-
-            before_decimal_pt = false;
-            continue;
-        }
-
-        if (has_digit && (*p == 'e' || *p == 'E'))
-        {
-            ++p;
-            double extra_divisor = parse_exponent(p, p_end);
-            if (extra_divisor)
-                divisor *= extra_divisor;
-            break;
-        }
-
-        if (*p < '0' || '9' < *p)
-        {
-            ret /= divisor;
-            return negative_sign ? -ret : ret;
-        }
-
-        has_digit = true;
-        ret *= 10.0;
-        ret += *p - '0';
-
-        if (!before_decimal_pt)
-            divisor *= 10.0;
-    }
-
-    ret /= divisor;
-    return negative_sign ? -ret : ret;
+    numeric_parser_type parser(p, p_end);
+    double v = parser.parse();
+    if (!std::isnan(v))
+        p = parser.get_char_position();
+    return v;
 }
 
 long parse_integer(const char*& p, size_t max_length)
@@ -257,6 +155,7 @@ parse_quoted_string_state parse_string_with_escaped_char(
     ret.str = nullptr;
     ret.length = 0;
     ret.transient = true;
+    ret.has_control_character = false;
 
     // Start the buffer with the string we've parsed so far.
     buffer.reset();
@@ -491,29 +390,23 @@ const char* parse_to_closing_single_quote(const char* p, size_t max_length)
 parse_quoted_string_state parse_double_quoted_string(
     const char*& p, size_t max_length, cell_buffer& buffer)
 {
-    assert(*p == '"');
+    if (max_length == 0 || !p || *p != '"')
+        throw invalid_arg_error("parse_double_quoted_string: invalid input string");
 
     parse_quoted_string_state ret;
     ret.str = nullptr;
     ret.length = 0;
     ret.transient = false;
-
-    if (max_length == 0)
-    {
-        ret.str = nullptr;
-        ret.length = parse_quoted_string_state::error_no_closing_quote;
-        return ret;
-    }
+    ret.has_control_character = false;
 
     const char* p_end = p + max_length;
-    ++p;
+    ++p; // skip the opening quote.
 
     ret.str = p;
-    ret.length = 0;
-    ret.transient = false;
 
     if (p == p_end)
     {
+        // The string contains only the opening quote.
         ret.str = nullptr;
         ret.length = parse_quoted_string_state::error_no_closing_quote;
         return ret;
@@ -523,9 +416,10 @@ parse_quoted_string_state parse_double_quoted_string(
 
     for (; p != p_end; ++p, ++ret.length)
     {
+        char c = *p;
+
         if (escape)
         {
-            char c = *p;
             escape = false;
 
             switch (get_string_escape_char_type(c))
@@ -546,19 +440,25 @@ parse_quoted_string_state parse_double_quoted_string(
         switch (*p)
         {
             case '"':
+            {
                 // closing quote.
                 ++p; // skip the quote.
                 return ret;
-            break;
+            }
             case '\\':
             {
                 escape = true;
                 continue;
             }
-            break;
             default:
                 ;
         }
+
+        if (0x00 <= c && c <= 0x1F)
+        {
+            // This is an unescaped control character.
+            ret.has_control_character = true;
+        }
     }
 
     ret.str = nullptr;
