@@ -12,6 +12,10 @@
 #include <vector>
 #include <memory>
 
+#ifdef __ORCUS_CPU_FEATURES
+#include <immintrin.h>
+#endif
+
 namespace orcus { namespace sax {
 
 malformed_xml_error::malformed_xml_error(const std::string& msg, std::ptrdiff_t offset) :
@@ -108,8 +112,8 @@ struct parser_base::impl
     std::vector<std::unique_ptr<cell_buffer>> m_cell_buffers;
 };
 
-parser_base::parser_base(const char* content, size_t size) :
-    ::orcus::parser_base(content, size),
+parser_base::parser_base(const char* content, size_t size, bool transient_stream) :
+    ::orcus::parser_base(content, size, transient_stream),
     mp_impl(orcus::make_unique<impl>()),
     m_nest_level(0),
     m_buffer_pos(0),
@@ -132,19 +136,6 @@ cell_buffer& parser_base::get_cell_buffer()
     return *mp_impl->m_cell_buffers[m_buffer_pos];
 }
 
-void parser_base::blank()
-{
-    char c = cur_char_checked();
-    while (is_blank(c))
-    {
-        next();
-        if (!has_char())
-            return;
-
-        c = cur_char();
-    }
-}
-
 void parser_base::comment()
 {
     // Parse until we reach '-->'.
@@ -269,11 +260,10 @@ void parser_base::parse_encoded_char(cell_buffer& buf)
         "error parsing encoded character: terminating character is not found.", offset());
 }
 
-void parser_base::value_with_encoded_char(cell_buffer& buf, pstring& str)
+void parser_base::value_with_encoded_char(cell_buffer& buf, pstring& str, char quote_char)
 {
     assert(cur_char() == '&');
     parse_encoded_char(buf);
-    assert(cur_char() != ';');
 
     const char* p0 = mp_char;
 
@@ -288,7 +278,7 @@ void parser_base::value_with_encoded_char(cell_buffer& buf, pstring& str)
             p0 = mp_char;
         }
 
-        if (cur_char() == '"')
+        if (cur_char() == quote_char)
             break;
 
         if (cur_char() != '&')
@@ -302,7 +292,7 @@ void parser_base::value_with_encoded_char(cell_buffer& buf, pstring& str)
         str = pstring(buf.get(), buf.size());
 
     // Skip the closing quote.
-    assert(!has_char() || cur_char() == '"');
+    assert(!has_char() || cur_char() == quote_char);
     next();
 }
 
@@ -325,7 +315,7 @@ bool parser_base::value(pstring& str, bool decode)
             cell_buffer& buf = get_cell_buffer();
             buf.reset();
             buf.append(p0, mp_char-p0);
-            value_with_encoded_char(buf, str);
+            value_with_encoded_char(buf, str, quote_char);
             return true;
         }
     }
@@ -335,7 +325,7 @@ bool parser_base::value(pstring& str, bool decode)
     // Skip the closing quote.
     next();
 
-    return false;
+    return transient_stream();
 }
 
 void parser_base::name(pstring& str)
@@ -349,8 +339,33 @@ void parser_base::name(pstring& str)
         throw malformed_xml_error(os.str(), offset());
     }
 
+#if defined(__ORCUS_CPU_FEATURES) && defined(__SSE4_2__)
+
+    const __m128i match = _mm_loadu_si128((const __m128i*)"azAZ09--__");
+    const int mode = _SIDD_LEAST_SIGNIFICANT | _SIDD_CMP_RANGES | _SIDD_UBYTE_OPS | _SIDD_NEGATIVE_POLARITY;
+
+    size_t n_total = available_size();
+
+    while (n_total)
+    {
+        __m128i char_block = _mm_loadu_si128((const __m128i*)mp_char);
+
+        int n = std::min<size_t>(16u, n_total);
+        int r = _mm_cmpestri(match, 10, char_block, n, mode);
+        mp_char += r; // Move the current char position.
+
+        if (r < 16)
+            // No need to move to the next segment. Stop here.
+            break;
+
+        // Skip 16 chars to the next segment.
+        n_total -= 16;
+    }
+
+#else
     while (is_alpha(c) || is_numeric(c) || is_name_char(c))
         c = next_char_checked();
+#endif
 
     str = pstring(p0, mp_char-p0);
 }
