@@ -6,6 +6,7 @@
  */
 
 #include "ods.hpp"
+#include "global.hpp"
 
 #ifdef __ORCUS_PYTHON_ODS
 #include "document.hpp"
@@ -21,15 +22,26 @@ namespace orcus { namespace python {
 
 PyObject* ods_read(PyObject* /*module*/, PyObject* args, PyObject* kwargs)
 {
-    PyObject* obj_bytes = read_stream_object_from_args(args, kwargs);
-    if (!obj_bytes)
+    stream_with_formulas data = read_stream_and_formula_params_from_args(args, kwargs);
+    if (!data.stream)
         return nullptr;
 
-    std::unique_ptr<spreadsheet::document> doc = orcus::make_unique<spreadsheet::document>();
-    spreadsheet::import_factory fact(*doc);
-    orcus_ods app(&fact);
+    try
+    {
+        spreadsheet::range_size_t ss{1048576, 16384};
+        std::unique_ptr<spreadsheet::document> doc = orcus::make_unique<spreadsheet::document>(ss);
+        spreadsheet::import_factory fact(*doc);
+        fact.set_recalc_formula_cells(data.recalc_formula_cells);
+        fact.set_formula_error_policy(data.error_policy);
+        orcus_ods app(&fact);
 
-    return import_from_stream_into_document(obj_bytes, app, std::move(doc));
+        return import_from_stream_into_document(data.stream.get(), app, std::move(doc));
+    }
+    catch (const std::exception& e)
+    {
+        set_python_exception(PyExc_RuntimeError, e);
+        return nullptr;
+    }
 }
 
 #else
