@@ -7,21 +7,27 @@
 
 #include "sheet.hpp"
 #include "sheet_rows.hpp"
+#include "named_expression.hpp"
+#include "named_expressions.hpp"
 
 #include "orcus/spreadsheet/types.hpp"
 #include "orcus/spreadsheet/sheet.hpp"
 #include "orcus/spreadsheet/document.hpp"
 
+#include <ixion/model_context.hpp>
+#include <ixion/named_expressions_iterator.hpp>
+
 #include <structmember.h>
 #include <bytesobject.h>
 #include <iostream>
 #include <cstring>
 
+namespace ss = orcus::spreadsheet;
+
 namespace orcus { namespace python {
 
-sheet_data::~sheet_data()
-{
-}
+sheet_data::sheet_data() : m_doc(nullptr), m_sheet(nullptr) {}
+sheet_data::~sheet_data() {}
 
 namespace {
 
@@ -35,13 +41,19 @@ struct pyobj_sheet
     PyObject* name;
     PyObject* sheet_size;
     PyObject* data_size;
+    PyObject* named_expressions;
 
-    sheet_data* m_data;
+    sheet_data* data;
 };
 
+inline pyobj_sheet* t(PyObject* self)
+{
+    return reinterpret_cast<pyobj_sheet*>(self);
+}
+
 inline sheet_data* get_sheet_data(PyObject* self)
 {
-    return reinterpret_cast<pyobj_sheet*>(self)->m_data;
+    return t(self)->data;
 }
 
 /**
@@ -53,9 +65,9 @@ inline spreadsheet::sheet* get_core_sheet(PyObject* self)
     return get_sheet_data(self)->m_sheet;
 }
 
-void sheet_dealloc(pyobj_sheet* self)
+void tp_dealloc(pyobj_sheet* self)
 {
-    delete self->m_data;
+    delete self->data;
 
     Py_CLEAR(self->name);
     Py_CLEAR(self->sheet_size);
@@ -64,14 +76,14 @@ void sheet_dealloc(pyobj_sheet* self)
     Py_TYPE(self)->tp_free(reinterpret_cast<PyObject*>(self));
 }
 
-PyObject* sheet_new(PyTypeObject* type, PyObject* /*args*/, PyObject* /*kwargs*/)
+PyObject* tp_new(PyTypeObject* type, PyObject* /*args*/, PyObject* /*kwargs*/)
 {
     pyobj_sheet* self = (pyobj_sheet*)type->tp_alloc(type, 0);
-    self->m_data = new sheet_data;
+    self->data = new sheet_data;
     return reinterpret_cast<PyObject*>(self);
 }
 
-int sheet_init(pyobj_sheet* self, PyObject* /*args*/, PyObject* /*kwargs*/)
+int tp_init(pyobj_sheet* self, PyObject* /*args*/, PyObject* /*kwargs*/)
 {
     return 0;
 }
@@ -87,9 +99,9 @@ PyObject* sheet_get_rows(PyObject* self, PyObject* args, PyObject* kwargs)
     sr_type->tp_init(rows, nullptr, nullptr);
 
     // Populate the sheet rows data.
-    store_sheet_rows_data(rows, get_core_sheet(self));
+    sheet_data* data = get_sheet_data(self);
+    store_sheet_rows_data(rows, data->m_doc, data->m_sheet);
 
-    Py_INCREF(rows);
     return rows;
 }
 
@@ -133,6 +145,30 @@ format_t to_format_type_enum(PyObject* format)
     return format_t::csv;
 }
 
+bool sheet_write_as_csv(spreadsheet::sheet* sheet, PyObject* file)
+{
+    std::ostringstream os;
+    sheet->dump_csv(os);
+    std::string s = os.str();
+
+    if (!s.empty())
+    {
+        PyObject* func_write = PyObject_GetAttrString(file, "write"); // new reference
+        if (!func_write)
+        {
+            PyErr_SetString(PyExc_RuntimeError, "'write' function was expected, but not found.");
+            return false;
+        }
+
+        // write the content as python's utf-8 string ('s').  Use 'y' to write
+        // it as bytes (for future reference).
+        PyObject_CallFunction(func_write, "s", s.data(), nullptr);
+        Py_XDECREF(func_write);
+    }
+
+    return true;
+}
+
 }
 
 PyObject* sheet_write(PyObject* self, PyObject* args, PyObject* kwargs)
@@ -145,39 +181,50 @@ PyObject* sheet_write(PyObject* self, PyObject* args, PyObject* kwargs)
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OO", const_cast<char**>(kwlist), &file, &format))
         return nullptr;
 
-    if (to_format_type_enum(format) == format_t::unknown)
+    format_t ft = to_format_type_enum(format);
+
+    if (ft == format_t::unknown)
         return nullptr;
 
     spreadsheet::sheet* sheet = get_core_sheet(self);
 
-    std::ostringstream os;
-    sheet->dump_csv(os);
-    std::string s = os.str();
-
-    if (!s.empty())
+    switch (ft)
     {
-        PyObject* func_write = PyObject_GetAttrString(file, "write");
-        if (!func_write)
+        case format_t::csv:
         {
-            PyErr_SetString(PyExc_RuntimeError, "'write' function was expected, but not found.");
+            if (!sheet_write_as_csv(sheet, file))
+                return nullptr;
+
+            break;
+        }
+        default:
+        {
+            PyErr_SetString(PyExc_RuntimeError, "this format is not yet supported.");
             return nullptr;
         }
-
-        PyObject_CallFunction(func_write, "y", s.data(), nullptr);
     }
 
     Py_INCREF(Py_None);
     return Py_None;
 }
 
-PyMethodDef sheet_methods[] =
+PyObject* sheet_get_named_expressions(PyObject* self, PyObject* args, PyObject* kwargs)
+{
+    const ss::document& doc = *t(self)->data->m_doc;
+    ss::sheet_t si = t(self)->data->m_sheet->get_index();
+    const ixion::model_context& cxt = doc.get_model_context();
+    return create_named_expressions_object(si, doc, cxt.get_named_expressions_iterator(si));
+}
+
+PyMethodDef tp_methods[] =
 {
     { "get_rows", (PyCFunction)sheet_get_rows, METH_VARARGS, "Get a sheet row iterator." },
     { "write", (PyCFunction)sheet_write, METH_VARARGS | METH_KEYWORDS, "Write sheet content to specified file object." },
+    { "get_named_expressions", (PyCFunction)sheet_get_named_expressions, METH_NOARGS, "Get a named expressions iterator." },
     { nullptr }
 };
 
-PyMemberDef sheet_members[] =
+PyMemberDef tp_members[] =
 {
     { (char*)"name",       T_OBJECT_EX, offsetof(pyobj_sheet, name),       READONLY, (char*)"sheet name" },
     { (char*)"sheet_size", T_OBJECT_EX, offsetof(pyobj_sheet, sheet_size), READONLY, (char*)"sheet size" },
@@ -189,9 +236,9 @@ PyTypeObject sheet_type =
 {
     PyVarObject_HEAD_INIT(nullptr, 0)
     "orcus.Sheet",                            // tp_name
-    sizeof(pyobj_sheet),                            // tp_basicsize
+    sizeof(pyobj_sheet),                      // tp_basicsize
     0,                                        // tp_itemsize
-    (destructor)sheet_dealloc,                // tp_dealloc
+    (destructor)tp_dealloc,                   // tp_dealloc
     0,                                        // tp_print
     0,                                        // tp_getattr
     0,                                        // tp_setattr
@@ -214,24 +261,24 @@ PyTypeObject sheet_type =
     0,		                                  // tp_weaklistoffset
     0,		                                  // tp_iter
     0,		                                  // tp_iternext
-    sheet_methods,                            // tp_methods
-    sheet_members,                            // tp_members
+    tp_methods,                               // tp_methods
+    tp_members,                               // tp_members
     0,                                        // tp_getset
     0,                                        // tp_base
     0,                                        // tp_dict
     0,                                        // tp_descr_get
     0,                                        // tp_descr_set
     0,                                        // tp_dictoffset
-    (initproc)sheet_init,                     // tp_init
+    (initproc)tp_init,                     // tp_init
     0,                                        // tp_alloc
-    sheet_new,                                // tp_new
+    tp_new,                                // tp_new
 };
 
 }
 
 sheet_data* get_sheet_data(PyObject* self)
 {
-    return reinterpret_cast<pyobj_sheet*>(self)->m_data;
+    return reinterpret_cast<pyobj_sheet*>(self)->data;
 }
 
 PyTypeObject* get_sheet_type()
@@ -240,17 +287,18 @@ PyTypeObject* get_sheet_type()
 }
 
 void store_sheet(
-    PyObject* self, const spreadsheet::document& doc, spreadsheet::sheet* orcus_sheet)
+    PyObject* self, const spreadsheet::document* doc, spreadsheet::sheet* orcus_sheet)
 {
     pyobj_sheet* pysheet = reinterpret_cast<pyobj_sheet*>(self);
-    pysheet->m_data->m_sheet = orcus_sheet;
+    pysheet->data->m_doc = doc;
+    pysheet->data->m_sheet = orcus_sheet;
 
     // Populate the python members.
 
     // Sheet name
     spreadsheet::sheet_t sid = orcus_sheet->get_index();
-    pstring name = doc.get_sheet_name(sid);
-    pysheet->name = PyUnicode_FromStringAndSize(name.get(), name.size());
+    pstring sheet_name = doc->get_sheet_name(sid);
+    pysheet->name = PyUnicode_FromStringAndSize(sheet_name.data(), sheet_name.size());
 
     // Data size - size of the data area.
     ixion::abs_range_t range = orcus_sheet->get_data_range();
@@ -268,8 +316,9 @@ void store_sheet(
 
     // Sheet size - size of the entire sheet.
     pysheet->sheet_size = PyDict_New();
-    PyDict_SetItemString(pysheet->sheet_size, "column", PyLong_FromLong(orcus_sheet->col_size()));
-    PyDict_SetItemString(pysheet->sheet_size, "row", PyLong_FromLong(orcus_sheet->row_size()));
+    ss::range_size_t sheet_size = doc->get_sheet_size();
+    PyDict_SetItemString(pysheet->sheet_size, "column", PyLong_FromLong(sheet_size.columns));
+    PyDict_SetItemString(pysheet->sheet_size, "row", PyLong_FromLong(sheet_size.rows));
 }
 
 }}
