@@ -40,6 +40,16 @@ namespace orcus { namespace spreadsheet {
 
 namespace {
 
+void init_once()
+{
+    static bool initialized = false;
+    if (initialized)
+        return;
+
+    ixion::init();
+    initialized = true;
+}
+
 /**
  * Single sheet entry which consists of a sheet name and a sheet data.
  */
@@ -50,13 +60,13 @@ struct sheet_item
 
     pstring name;
     sheet   data;
-    sheet_item(document& doc, const pstring& _name, sheet_t sheet_index, row_t row_size, col_t col_size);
+    sheet_item(document& doc, const pstring& _name, sheet_t sheet_index);
 };
 
 typedef std::map<pstring, std::unique_ptr<table_t>> table_store_type;
 
-sheet_item::sheet_item(document& doc, const pstring& _name, sheet_t sheet_index, row_t row_size, col_t col_size) :
-    name(_name), data(doc, sheet_index, row_size, col_size) {}
+sheet_item::sheet_item(document& doc, const pstring& _name, sheet_t sheet_index) :
+    name(_name), data(doc, sheet_index) {}
 
 class find_sheet_by_name : std::unary_function<std::unique_ptr<sheet_item> , bool>
 {
@@ -293,21 +303,25 @@ struct document_impl
 
     pivot_collection m_pivots;
 
-    std::unique_ptr<ixion::formula_name_resolver> mp_name_resolver;
+    std::unique_ptr<ixion::formula_name_resolver> mp_name_resolver_global;
+    std::unique_ptr<ixion::formula_name_resolver> mp_name_resolver_named_exp_base;
+    std::unique_ptr<ixion::formula_name_resolver> mp_name_resolver_named_range;
     formula_grammar_t m_grammar;
 
     table_store_type m_tables;
     table_handler m_table_handler;
 
-    document_impl(document& doc) :
+    document_impl(document& doc, const range_size_t& sheet_size) :
         m_doc(doc),
+        m_context({sheet_size.rows, sheet_size.columns}),
         m_styles(),
         mp_strings(new import_shared_strings(m_string_pool, m_context, m_styles)),
         m_pivots(doc),
-        mp_name_resolver(ixion::formula_name_resolver::get(ixion::formula_name_resolver_t::excel_a1, &m_context)),
+        mp_name_resolver_global(ixion::formula_name_resolver::get(ixion::formula_name_resolver_t::excel_a1, &m_context)),
         m_grammar(formula_grammar_t::xlsx),
         m_table_handler(m_context, m_tables)
     {
+        init_once();
         m_context.set_table_handler(&m_table_handler);
     }
 
@@ -317,7 +331,7 @@ struct document_impl
     }
 };
 
-document::document() : mp_impl(new document_impl(*this)) {}
+document::document(const range_size_t& sheet_size) : mp_impl(new document_impl(*this, sheet_size)) {}
 
 document::~document() {}
 
@@ -403,21 +417,17 @@ void document::finalize()
             sh->data.finalize();
         }
     );
-
-    calc_formulas();
 }
 
-sheet* document::append_sheet(const pstring& sheet_name, row_t row_size, col_t col_size)
+sheet* document::append_sheet(const pstring& sheet_name)
 {
     pstring sheet_name_safe = mp_impl->m_string_pool.intern(sheet_name).first;
     sheet_t sheet_index = static_cast<sheet_t>(mp_impl->m_sheets.size());
 
     mp_impl->m_sheets.push_back(
-        orcus::make_unique<sheet_item>(
-            *this, sheet_name_safe, sheet_index, row_size, col_size));
+        orcus::make_unique<sheet_item>(*this, sheet_name_safe, sheet_index));
 
-    mp_impl->m_context.append_sheet(
-        sheet_name_safe.get(), sheet_name_safe.size(), row_size, col_size);
+    mp_impl->m_context.append_sheet(sheet_name_safe.get(), sheet_name_safe.size());
 
     return &mp_impl->m_sheets.back()->data;
 }
@@ -453,7 +463,7 @@ const sheet* document::get_sheet(sheet_t sheet_pos) const
     return &mp_impl->m_sheets[sheet_pos]->data;
 }
 
-void document::calc_formulas()
+void document::recalc_formula_cells()
 {
     ixion::abs_range_set_t empty;
 
@@ -465,7 +475,7 @@ void document::calc_formulas()
 
 void document::clear()
 {
-    mp_impl.reset(new document_impl(*this));
+    mp_impl.reset(new document_impl(*this, get_sheet_size()));
 }
 
 void document::dump(dump_format_t format, const std::string& output) const
@@ -654,7 +664,21 @@ pstring document::get_sheet_name(sheet_t sheet_pos) const
     return mp_impl->m_sheets[pos]->name;
 }
 
-size_t document::sheet_size() const
+range_size_t document::get_sheet_size() const
+{
+    ixion::rc_size_t ss = mp_impl->m_context.get_sheet_size();
+    range_size_t ret;
+    ret.rows = ss.row;
+    ret.columns = ss.column;
+    return ret;
+}
+
+void document::set_sheet_size(const range_size_t& sheet_size)
+{
+    mp_impl->m_context.set_sheet_size({sheet_size.rows, sheet_size.columns});
+}
+
+size_t document::get_sheet_count() const
 {
     return mp_impl->m_sheets.size();
 }
@@ -678,38 +702,55 @@ void document::set_formula_grammar(formula_grammar_t grammar)
 
     mp_impl->m_grammar = grammar;
 
-    ixion::formula_name_resolver_t resolver_type = ixion::formula_name_resolver_t::unknown;
+    ixion::formula_name_resolver_t resolver_type_global = ixion::formula_name_resolver_t::unknown;
+    ixion::formula_name_resolver_t resolver_type_named_exp_base = ixion::formula_name_resolver_t::unknown;
+    ixion::formula_name_resolver_t resolver_type_named_range = ixion::formula_name_resolver_t::unknown;
     char arg_sep = 0;
 
     switch (mp_impl->m_grammar)
     {
         case formula_grammar_t::xls_xml:
-            resolver_type = ixion::formula_name_resolver_t::excel_r1c1;
+            resolver_type_global = ixion::formula_name_resolver_t::excel_r1c1;
             arg_sep = ',';
             break;
         case formula_grammar_t::xlsx:
-            resolver_type = ixion::formula_name_resolver_t::excel_a1;
+            resolver_type_global = ixion::formula_name_resolver_t::excel_a1;
             arg_sep = ',';
             break;
         case formula_grammar_t::ods:
-            resolver_type = ixion::formula_name_resolver_t::odff;
+            resolver_type_global = ixion::formula_name_resolver_t::odff;
+            resolver_type_named_exp_base = ixion::formula_name_resolver_t::calc_a1;
+            resolver_type_named_range = ixion::formula_name_resolver_t::odf_cra;
             arg_sep = ';';
             break;
         case formula_grammar_t::gnumeric:
             // TODO : Use Excel A1 name resolver for now.
-            resolver_type = ixion::formula_name_resolver_t::excel_a1;
+            resolver_type_global = ixion::formula_name_resolver_t::excel_a1;
             arg_sep = ',';
             break;
         default:
-            mp_impl->mp_name_resolver.reset();
+            ;
     }
 
-    mp_impl->mp_name_resolver.reset();
+    mp_impl->mp_name_resolver_global.reset();
+    mp_impl->mp_name_resolver_named_exp_base.reset();
 
-    if (resolver_type != ixion::formula_name_resolver_t::unknown)
+    if (resolver_type_global != ixion::formula_name_resolver_t::unknown)
     {
-        mp_impl->mp_name_resolver =
-            ixion::formula_name_resolver::get(resolver_type, &mp_impl->m_context);
+        mp_impl->mp_name_resolver_global =
+            ixion::formula_name_resolver::get(resolver_type_global, &mp_impl->m_context);
+
+        if (resolver_type_named_exp_base != ixion::formula_name_resolver_t::unknown)
+        {
+            mp_impl->mp_name_resolver_named_exp_base =
+                ixion::formula_name_resolver::get(resolver_type_named_exp_base, &mp_impl->m_context);
+        }
+
+        if (resolver_type_named_range != ixion::formula_name_resolver_t::unknown)
+        {
+            mp_impl->mp_name_resolver_named_range =
+                ixion::formula_name_resolver::get(resolver_type_named_range, &mp_impl->m_context);
+        }
 
         ixion::config cfg = mp_impl->m_context.get_config();
         cfg.sep_function_arg = arg_sep;
@@ -723,9 +764,25 @@ formula_grammar_t document::get_formula_grammar() const
     return mp_impl->m_grammar;
 }
 
-const ixion::formula_name_resolver* document::get_formula_name_resolver() const
+const ixion::formula_name_resolver* document::get_formula_name_resolver(formula_ref_context_t cxt) const
 {
-    return mp_impl->mp_name_resolver.get();
+    switch (cxt)
+    {
+        case formula_ref_context_t::global:
+            return mp_impl->mp_name_resolver_global.get();
+        case formula_ref_context_t::named_expression_base:
+            if (mp_impl->mp_name_resolver_named_exp_base)
+                return mp_impl->mp_name_resolver_named_exp_base.get();
+            break;
+        case formula_ref_context_t::named_range:
+            if (mp_impl->mp_name_resolver_named_range)
+                return mp_impl->mp_name_resolver_named_range.get();
+            break;
+        default:
+            ;
+    }
+
+    return mp_impl->mp_name_resolver_global.get();
 }
 
 void document::insert_dirty_cell(const ixion::abs_address_t& pos)
