@@ -67,6 +67,20 @@ typedef mdds::flat_segment_tree<row_t, row_height_t> row_heights_store_type;
 typedef mdds::flat_segment_tree<col_t, bool> col_hidden_store_type;
 typedef mdds::flat_segment_tree<row_t, bool> row_hidden_store_type;
 
+ixion::abs_range_t to_ixion_range(sheet_t sheet, const range_t& range)
+{
+    ixion::abs_range_t pos;
+
+    pos.first.sheet  = sheet;
+    pos.first.row    = range.first.row;
+    pos.first.column = range.first.column;
+    pos.last.sheet   = sheet;
+    pos.last.row     = range.last.row;
+    pos.last.column  = range.last.column;
+
+    return pos;
+}
+
 }
 
 struct sheet_impl
@@ -88,25 +102,23 @@ struct sheet_impl
     std::unique_ptr<auto_filter_t> mp_auto_filter_data;
 
     cell_format_type m_cell_formats;
-    row_t m_row_size;
-    col_t m_col_size;
     const sheet_t m_sheet; /// sheet ID
 
     sheet_impl() = delete;
     sheet_impl(const sheet_impl&) = delete;
     sheet_impl& operator=(const sheet_impl&) = delete;
 
-    sheet_impl(document& doc, sheet& sh, sheet_t sheet_index, row_t row_size, col_t col_size) :
+    sheet_impl(document& doc, sheet& sh, sheet_t sheet_index) :
         m_doc(doc),
-        m_col_widths(0, col_size, get_default_column_width()),
-        m_row_heights(0, row_size, get_default_row_height()),
+        m_col_widths(0, m_doc.get_sheet_size().columns, get_default_column_width()),
+        m_row_heights(0, m_doc.get_sheet_size().rows, get_default_row_height()),
         m_col_width_pos(m_col_widths.begin()),
         m_row_height_pos(m_row_heights.begin()),
-        m_col_hidden(0, col_size, false),
-        m_row_hidden(0, row_size, false),
+        m_col_hidden(0, m_doc.get_sheet_size().columns, false),
+        m_row_hidden(0, m_doc.get_sheet_size().rows, false),
         m_col_hidden_pos(m_col_hidden.begin()),
         m_row_hidden_pos(m_row_hidden.begin()),
-        m_row_size(row_size), m_col_size(col_size), m_sheet(sheet_index) {}
+        m_sheet(sheet_index) {}
 
     ~sheet_impl() {}
 
@@ -134,8 +146,8 @@ struct sheet_impl
 const row_t sheet::max_row_limit = 1048575;
 const col_t sheet::max_col_limit = 1023;
 
-sheet::sheet(document& doc, sheet_t sheet_index, row_t row_size, col_t col_size) :
-    mp_impl(new sheet_impl(doc, *this, sheet_index, row_size, col_size)) {}
+sheet::sheet(document& doc, sheet_t sheet_index) :
+    mp_impl(new sheet_impl(doc, *this, sheet_index)) {}
 
 sheet::~sheet()
 {
@@ -167,7 +179,7 @@ void sheet::set_string(row_t row, col_t col, size_t sindex)
     cxt.set_string_cell(ixion::abs_address_t(mp_impl->m_sheet,row,col), sindex);
 
 #if ORCUS_DEBUG_SHEET
-    cout << "sheet::set_string: sheet=" << mp_impl->m_sheet << ", row=" << row << ", col=" << col << ", si=" << sindex << endl;
+    cout << "sheet::set_string: sheet=" << mp_impl->m_sheet << "; row=" << row << "; col=" << col << "; si=" << sindex << endl;
 #endif
 }
 
@@ -221,7 +233,7 @@ void sheet::set_format(row_t row_start, col_t col_start, row_t row_end, col_t co
         cell_format_type::iterator itr = mp_impl->m_cell_formats.find(col);
         if (itr == mp_impl->m_cell_formats.end())
         {
-            auto p = orcus::make_unique<segment_row_index_type>(0, mp_impl->m_row_size+1, 0);
+            auto p = orcus::make_unique<segment_row_index_type>(0, mp_impl->m_doc.get_sheet_size().rows+1, 0);
 
             pair<cell_format_type::iterator, bool> r =
                 mp_impl->m_cell_formats.insert(cell_format_type::value_type(col, std::move(p)));
@@ -246,33 +258,77 @@ void sheet::set_formula(row_t row, col_t col, const ixion::formula_tokens_store_
     ixion::abs_address_t pos(mp_impl->m_sheet, row, col);
 
     cxt.set_formula_cell(pos, tokens);
-    ixion::register_formula_cell(cxt, pos);
-    mp_impl->m_doc.insert_dirty_cell(pos);
+    try
+    {
+        ixion::register_formula_cell(cxt, pos);
+        mp_impl->m_doc.insert_dirty_cell(pos);
+    }
+    catch (const ixion::formula_registration_error& e)
+    {
+#if ORCUS_DEBUG_SHEET
+        cout << "sheet::set_formula: sheet=" << mp_impl->m_sheet << "; row=" << row << "; col=" << col << "; e=" << e.what() << endl;
+#endif
+    }
 }
 
-void sheet::set_grouped_formula(const range_t range, ixion::formula_tokens_t tokens)
+void sheet::set_formula(
+    row_t row, col_t col, const ixion::formula_tokens_store_ptr_t& tokens,
+    ixion::formula_result result)
 {
     ixion::model_context& cxt = mp_impl->m_doc.get_model_context();
+    ixion::abs_address_t pos(mp_impl->m_sheet, row, col);
 
-    ixion::abs_range_t pos;
-    pos.first.sheet  = mp_impl->m_sheet;
-    pos.first.row    = range.first.row;
-    pos.first.column = range.first.column;
-    pos.last.sheet   = mp_impl->m_sheet;
-    pos.last.row     = range.last.row;
-    pos.last.column  = range.last.column;
+    cxt.set_formula_cell(pos, tokens, result);
+
+    try
+    {
+        ixion::register_formula_cell(cxt, pos);
+        mp_impl->m_doc.insert_dirty_cell(pos);
+    }
+    catch (const ixion::formula_registration_error& e)
+    {
+#if ORCUS_DEBUG_SHEET
+        cout << "sheet::set_formula: sheet=" << mp_impl->m_sheet << "; row=" << row << "; col=" << col << "; e=" << e.what() << endl;
+#endif
+    }
+}
+
+void sheet::set_grouped_formula(const range_t& range, ixion::formula_tokens_t tokens)
+{
+    ixion::abs_range_t pos = to_ixion_range(mp_impl->m_sheet, range);
+    ixion::model_context& cxt = mp_impl->m_doc.get_model_context();
 
     cxt.set_grouped_formula_cells(pos, std::move(tokens));
-    ixion::register_formula_cell(cxt, pos.first);
-    mp_impl->m_doc.insert_dirty_cell(pos.first);
+    try
+    {
+        ixion::register_formula_cell(cxt, pos.first);
+        mp_impl->m_doc.insert_dirty_cell(pos.first);
+    }
+    catch (const ixion::formula_registration_error& e)
+    {
+#if ORCUS_DEBUG_SHEET
+        cout << "sheet::set_formula: sheet=" << mp_impl->m_sheet << "; range=" << range << "; e=" << e.what() << endl;
+#endif
+    }
 }
 
-void sheet::set_formula_result(row_t row, col_t col, double value)
+void sheet::set_grouped_formula(const range_t& range, ixion::formula_tokens_t tokens, ixion::formula_result result)
 {
-}
+    ixion::abs_range_t pos = to_ixion_range(mp_impl->m_sheet, range);
+    ixion::model_context& cxt = mp_impl->m_doc.get_model_context();
 
-void sheet::set_formula_result(row_t row, col_t col, const char* p, size_t n)
-{
+    cxt.set_grouped_formula_cells(pos, std::move(tokens), std::move(result));
+    try
+    {
+        ixion::register_formula_cell(cxt, pos.first);
+        mp_impl->m_doc.insert_dirty_cell(pos.first);
+    }
+    catch (const ixion::formula_registration_error& e)
+    {
+#if ORCUS_DEBUG_SHEET
+        cout << "sheet::set_formula: sheet=" << mp_impl->m_sheet << "; range=" << range << "; e=" << e.what() << endl;
+#endif
+    }
 }
 
 void sheet::set_col_width(col_t col, col_width_t width)
@@ -452,16 +508,6 @@ sheet_range sheet::get_sheet_range(
     return sheet_range(cxt, *stores, row_start, col_start, row_end, col_end);
 }
 
-row_t sheet::row_size() const
-{
-    return mp_impl->m_row_size;
-}
-
-col_t sheet::col_size() const
-{
-    return mp_impl->m_col_size;
-}
-
 sheet_t sheet::get_index() const
 {
     return mp_impl->m_sheet;
@@ -582,7 +628,8 @@ void sheet::dump_check(ostream& os, const pstring& sheet_name) const
         return;
 
     const ixion::model_context& cxt = mp_impl->m_doc.get_model_context();
-    const ixion::formula_name_resolver* resolver = mp_impl->m_doc.get_formula_name_resolver();
+    const ixion::formula_name_resolver* resolver =
+        mp_impl->m_doc.get_formula_name_resolver(spreadsheet::formula_ref_context_t::global);
 
     size_t row_count = range.last.row + 1;
     size_t col_count = range.last.column + 1;
@@ -648,8 +695,16 @@ void sheet::dump_check(ostream& os, const pstring& sheet_name) const
                         else
                             os << formula;
 
-                        ixion::formula_result res = cell->get_result_cache();
-                        os << ':' << res.str(mp_impl->m_doc.get_model_context());
+                        try
+                        {
+                            ixion::formula_result res = cell->get_result_cache(
+                                ixion::formula_result_wait_policy_t::throw_exception);
+                            os << ':' << res.str(mp_impl->m_doc.get_model_context());
+                        }
+                        catch (const std::exception&)
+                        {
+                            os << ":#RES!";
+                        }
                     }
 
                     os << endl;
